#ifndef __ASM_ARM_IRQFLAGS_H
#define __ASM_ARM_IRQFLAGS_H

#ifdef __KERNEL__

#include <asm/ptrace.h>

/*
 * CPU interrupt mask handling.
 */
#ifdef CONFIG_LINUX_CLI_MASK
#include <mach/intctl-regs.h>

#define calc_flags_intc_pri(flags, pri)	\
	(((flags) & ~PSR_PRI_BIT) | (((pri) << PSR_PRI_SHIFT) & PSR_PRI_BIT))
#define get_flags_intc_pri(flags)	\
	(((flags) & PSR_PRI_BIT) >> PSR_PRI_SHIFT)

extern unsigned int __mn_intc_get_mask_priority(void);
extern unsigned int mn_intc_set_and_get_mask_priority(unsigned int pri);
extern void mn_intc_set_mask_priority(unsigned int pri);
extern unsigned int mn_intc_get_mask_priority(void);

#define raw_local_irq_save(x)					\
	({							\
	__asm__ __volatile__(					\
	"mrs	%0, cpsr		@ local_irq_save\n"	\
	: "=r" (x) : : "memory", "cc");				\
	x = calc_flags_intc_pri(x, mn_intc_set_and_get_mask_priority(CONFIG_LINUX_CLI_LEVEL));	\
	})

#define raw_local_irq_enable()  mn_intc_set_mask_priority(CONFIG_LINUX_TASK_LEVEL)
#define raw_local_irq_disable() mn_intc_set_mask_priority(CONFIG_LINUX_CLI_LEVEL)
#define local_fiq_enable()  __asm__("cpsie f	@ __stf" : : : "memory", "cc")
#define local_fiq_disable() __asm__("cpsid f	@ __clf" : : : "memory", "cc")

#define raw_local_save_flags(x)					\
	({							\
	__asm__ __volatile__(					\
	"mrs	%0, cpsr		@ local_irq_save\n"	\
	: "=r" (x) : : "memory", "cc");				\
	x = calc_flags_intc_pri(x, mn_intc_get_mask_priority());	\
	})

#define raw_local_irq_restore(x)	\
	mn_intc_set_mask_priority(get_flags_intc_pri(x))

#define raw_irqs_disabled_flags(flags)				\
	(get_flags_intc_pri(flags) != CONFIG_LINUX_TASK_LEVEL ?	\
	 1 : (int)((flags) & PSR_I_BIT))

#define local_fastintr_save(x)					\
	({							\
	__asm__ __volatile__(					\
	"mrs	%0, cpsr		@ local_irq_save\n"	\
	"cpsid	i"						\
	: "=r" (x) : : "memory", "cc");				\
	x = calc_flags_intc_pri(x, __mn_intc_get_mask_priority());	\
	})

#define local_fastintr_enable()  __asm__("cpsie i	@ __sti" : : : "memory", "cc")
#define local_fastintr_disable() __asm__("cpsid i	@ __cli" : : : "memory", "cc")

/*
 * restore saved IRQ & FIQ state
 */
#define local_fastintr_restore(x)				\
	__asm__ __volatile__(					\
	"msr	cpsr_c, %0		@ local_sti_restore\n"	\
	:							\
	: "r" ((x) & ~PSR_PRI_BIT)				\
	: "memory", "cc")

#else /* CONFIG_LINUX_CLI_MASK */
#if __LINUX_ARM_ARCH__ >= 6

#define raw_local_irq_save(x)					\
	({							\
	__asm__ __volatile__(					\
	"mrs	%0, cpsr		@ local_irq_save\n"	\
	"cpsid	i"						\
	: "=r" (x) : : "memory", "cc");				\
	})

#define raw_local_irq_enable()  __asm__("cpsie i	@ __sti" : : : "memory", "cc")
#define raw_local_irq_disable() __asm__("cpsid i	@ __cli" : : : "memory", "cc")
#define local_fiq_enable()  __asm__("cpsie f	@ __stf" : : : "memory", "cc")
#define local_fiq_disable() __asm__("cpsid f	@ __clf" : : : "memory", "cc")

#else

/*
 * Save the current interrupt enable state & disable IRQs
 */
#define raw_local_irq_save(x)					\
	({							\
		unsigned long temp;				\
		(void) (&temp == &x);				\
	__asm__ __volatile__(					\
	"mrs	%0, cpsr		@ local_irq_save\n"	\
"	orr	%1, %0, #128\n"					\
"	msr	cpsr_c, %1"					\
	: "=r" (x), "=r" (temp)					\
	:							\
	: "memory", "cc");					\
	})
	
/*
 * Enable IRQs
 */
#define raw_local_irq_enable()					\
	({							\
		unsigned long temp;				\
	__asm__ __volatile__(					\
	"mrs	%0, cpsr		@ local_irq_enable\n"	\
"	bic	%0, %0, #128\n"					\
"	msr	cpsr_c, %0"					\
	: "=r" (temp)						\
	:							\
	: "memory", "cc");					\
	})

/*
 * Disable IRQs
 */
#define raw_local_irq_disable()					\
	({							\
		unsigned long temp;				\
	__asm__ __volatile__(					\
	"mrs	%0, cpsr		@ local_irq_disable\n"	\
"	orr	%0, %0, #128\n"					\
"	msr	cpsr_c, %0"					\
	: "=r" (temp)						\
	:							\
	: "memory", "cc");					\
	})

/*
 * Enable FIQs
 */
#define local_fiq_enable()					\
	({							\
		unsigned long temp;				\
	__asm__ __volatile__(					\
	"mrs	%0, cpsr		@ stf\n"		\
"	bic	%0, %0, #64\n"					\
"	msr	cpsr_c, %0"					\
	: "=r" (temp)						\
	:							\
	: "memory", "cc");					\
	})

/*
 * Disable FIQs
 */
#define local_fiq_disable()					\
	({							\
		unsigned long temp;				\
	__asm__ __volatile__(					\
	"mrs	%0, cpsr		@ clf\n"		\
"	orr	%0, %0, #64\n"					\
"	msr	cpsr_c, %0"					\
	: "=r" (temp)						\
	:							\
	: "memory", "cc");					\
	})

#endif

/*
 * Save the current interrupt enable state.
 */
#define raw_local_save_flags(x)					\
	({							\
	__asm__ __volatile__(					\
	"mrs	%0, cpsr		@ local_save_flags"	\
	: "=r" (x) : : "memory", "cc");				\
	})

/*
 * restore saved IRQ & FIQ state
 */
#define raw_local_irq_restore(x)				\
	__asm__ __volatile__(					\
	"msr	cpsr_c, %0		@ local_irq_restore\n"	\
	:							\
	: "r" (x)						\
	: "memory", "cc")

#define raw_irqs_disabled_flags(flags)	\
({					\
	(int)((flags) & PSR_I_BIT);	\
})

#endif /* CONFIG_LINUX_CLI_MASK */
#endif
#endif
