#ifndef _xdmac_ioctl_h_included_
#define _xdmac_ioctl_h_included_

/* ============================================================= */
/*    include files                                              */
/* ============================================================= */


/* ============================================================= */
/*    macros (IOCTL)                                             */
/* ============================================================= */
#define XDMAC_IOCTL_MAGIC  0x31 /* magic number for /dev/xdmac */

#define XDMAC_IOCTL_INIT    _IOC(_IOC_NONE,  XDMAC_IOCTL_MAGIC, 0, 0 )
#define XDMAC_IOCTL_GET     _IOC(_IOC_READ,  XDMAC_IOCTL_MAGIC, 1, sizeof(int))
#define XDMAC_IOCTL_START   _IOC(_IOC_WRITE, XDMAC_IOCTL_MAGIC, 2, sizeof(int))
#define XDMAC_IOCTL_CONFIG  _IOC(_IOC_WRITE, XDMAC_IOCTL_MAGIC, 3, sizeof(int))
#define XDMAC_IOCTL_DCINV   _IOC(_IOC_WRITE, XDMAC_IOCTL_MAGIC, 4, sizeof(int))

/* ============================================================= */
/*    macros (IOCTL, internal use only)                          */
/* ============================================================= */
#define XDMAC_IOCTL_REGREAD  _IOC(_IOC_WRITE, XDMAC_IOCTL_MAGIC, 100, sizeof(int))
#define XDMAC_IOCTL_REGWRITE _IOC(_IOC_WRITE, XDMAC_IOCTL_MAGIC, 101, sizeof(int))


/* ============================================================= */
/*    macros                                                     */
/* ============================================================= */
#define XDMAC_ON  0
#define XDMAC_OFF 1


/* ============================================================= */
/*    structures, enums                                          */
/* ============================================================= */
typedef enum {
	kcs_8bit  = 0,
	kcs_16bit = 1,
	kcs_24bit = 2,
	kcs_32bit = 3
} EKcsType;

typedef enum {
	tfa_soft_startup    = 0x00,
	tfa_ch_ope_start    = 0x01,
	tfa_ch_ope_comp     = 0x02,
	tfa_s0_rsv_req      = 0x03,
	tfa_s0_snd_req      = 0x04,
	tfa_s1_rsv_req      = 0x05,
	tfa_s1_snd_req      = 0x06,
	tfa_s2_rsv_req      = 0x07,
	tfa_s2_snd_req      = 0x08,
	tfa_s3_rsv_req      = 0x09,
	tfa_s4_snd_req      = 0x0A,
	tfa_8bit_tim0_udf   = 0x0B,
	tfa_8bit_tim1_udf   = 0x0C,
	tfa_8bit_tim2_udf   = 0x0D,
	tfa_8bit_tim3_udf   = 0x0E,
	tfa_8bit_tim4_udf   = 0x0F,
	tfa_8bit_tim5_udf   = 0x10,
	tfa_8bit_tim6_udf   = 0x11,
	tfa_8bit_tim7_udf   = 0x12,
	tfa_8bit_tim8_udf   = 0x13,
	tfa_8bit_tim9_udf   = 0x14,
	tfa_8bit_tim10_udf  = 0x15,
	tfa_8bit_tim11_udf  = 0x16,
	tfa_8bit_tim12_udf  = 0x17,
	tfa_8bit_tim13_udf  = 0x18,
	tfa_8bit_tim14_udf  = 0x19,
	tfa_8bit_tim15_udf  = 0x1A,
	tfa_16bit_tim0_udf  = 0x1B,
	tfa_16bit_tim1_udf  = 0x1C,
	tfa_16bit_tim2_udf  = 0x1D,
	tfa_16bit_tim3_udf  = 0x1E,
	tfa_16bit_tim4_udf  = 0x1F,
	tfa_16bit_tim5_udf  = 0x20,
	tfa_16bit_tim6_udf  = 0x21,
	tfa_16bit_tim7_udf  = 0x22,
	tfa_16bit_tim8_udf  = 0x23,
	tfa_16bit_tim9_udf  = 0x24,
	tfa_16bit_tim10_udf = 0x25,
	tfa_16bit_tim11_udf = 0x26,
	tfa_16bit_tim0_CCA  = 0x27,
	tfa_16bit_tim0_CCB  = 0x28,
	tfa_16bit_tim1_CCA  = 0x29,
	tfa_16bit_tim1_CCB  = 0x2A,
	tfa_16bit_tim2_CCA  = 0x2B,
	tfa_16bit_tim2_CCB  = 0x2C,
	tfa_16bit_tim3_CCA  = 0x2D,
	tfa_16bit_tim3_CCB  = 0x2E,
	tfa_16bit_tim4_CCA  = 0x2F,
	tfa_16bit_tim4_CCB  = 0x30,
	tfa_16bit_tim5_CCA  = 0x31,
	tfa_16bit_tim5_CCB  = 0x32,
	tfa_16bit_tim6_CCA  = 0x33,
	tfa_16bit_tim6_CCB  = 0x34,
	tfa_16bit_tim7_CCA  = 0x35,
	tfa_16bit_tim7_CCB  = 0x36,
	tfa_16bit_tim8_CCA  = 0x37,
	tfa_16bit_tim8_CCB  = 0x38,
	tfa_16bit_tim9_CCA  = 0x39,
	tfa_16bit_tim9_CCB  = 0x3A,
	tfa_16bit_tim10_CCA = 0x3B,
	tfa_16bit_tim10_CCB = 0x3C,
	tfa_16bit_tim11_CCA = 0x3D,
	tfa_16bit_tim11_CCB = 0x3E,
	tfa_nif_int         = 0x3F,
	tfa_exirq3_pos      = 0x40,
	tfa_exirq3_neg      = 0x41,
	tfa_exirq4_pos      = 0x42,
	tfa_exirq4_neg      = 0x43,
	tfa_exirq8_pos      = 0x44,
	tfa_exirq8_neg      = 0x45,
	tfa_exirq9_pos      = 0x46,
	tfa_exirq9_neg      = 0x47,
	tfa_exirq10_pos     = 0x48,
	tfa_exirq10_neg     = 0x49,
	tfa_exirq11_pos     = 0x4A,
	tfa_exirq11_neg     = 0x4B
} ETfaType;

typedef enum {
	tw_1B = 0,
	tw_2B = 1,
	tw_4B = 2,
	tw_8B = 3
} ETwType;

typedef enum {
	et_LE = 0,
	et_32bit_HW_BE = 1,
	et_16bit_BE = 2,
	et_32bit_BE = 3
} EEndianType;

typedef enum {
	am_inc = 0,
	am_fix = 1
} EAmType;

typedef enum {
	tm_nrm = 0,
	tm_rct = 1
} ETmType;

typedef enum {
	pri_fix    = 0,
	pri_rndrbn = 1
} EPriType;

typedef enum {
	stat_trns = 0,
	stat_stop = 1,
	stat_err  = 2
} EStatType;

typedef struct _SDmaParam {
	char           cKcen;
	EKcsType       eKcs;
	ETfaType       eTfa;
	unsigned int uiSrcAdr;
	ETwType        eSrcTw;
	EEndianType    eSrcEndian;
	EAmType        eSrcAm;
	ETmType        eSrcTm;
	unsigned int uiDstAdr;
	ETwType        eDstTw;
	EEndianType    eDstEndian;
	EAmType        eDstAm;
	ETmType        eDstTm;
	unsigned int uiIts;
	unsigned int uiTnum;
	unsigned int uiSqs;
	unsigned int uiSqad2d;
	char          cWaitEvent;
	char          cInvCache;
} SDmaParam;

typedef struct _SDmaStat{
	EStatType eStat;
} SDmaStat;

typedef struct _SDmaCfg {
	EPriType ePri;
	unsigned int uiKcdata;
} SDmaCfg;

typedef struct _SDmaDstInfo {
	unsigned int	uiDstAdr;
	ETmType			eDstTm;
	unsigned int	uiSize;
	unsigned int	uiSqs;
	unsigned int	uiSqad2d;
} SDmaDstInfo;


/* ============================================================= */
/*    structures, enums(internal use only)                       */
/* ============================================================= */
typedef struct _SDmaRegInfo {
	unsigned int	uiAddr;
	unsigned int	uiData;
} SDmaRegInfo;


/* ============================================================= */
/*    API prototype definitions                                  */
/* ============================================================= */

#endif

