/*
 * arch/arm/mach-pvc01s/include/mach/platform.h
 *
 * Copyright (c) 2012 Panasonic, Inc
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#ifndef __ASM_ARCH_PLATFORM_H
#define __ASM_ARCH_PLATFORM_H


//
//	physical memory map
//
//	IRC GROUP reg
#define PVC01S_IRCGROUP_PHY_SIZE	0x1000
#define PVC01S_IRCGROUP_PHY_ADDR	0xffffe000


//	IRC : target = irc1
//
//	IRC(CPU0)	addr : 0xfffffa00
//				size : 0x200(512byte)
//	IRC(CPU1)	addr : 0xfffffc00
//				size : 0x200(512byte)
//	IRC(CPU2)	addr : 0xfffffe00
//				size : 0x200(512byte)
//#define PVC01S_IRC0_PHY_ADDR	0xfffffa00
#define PVC01S_IRC_MAPPED_TOP_PHY_SIZE	0x1000
#define PVC01S_IRC_MAPPED_TOP_PHY_ADDR	0xfffff000
#define PVC01S_IRC_PHY_SIZE			0x200
#define PVC01S_IRC0_PHY_ADDR		0xfffffa00
#define PVC01S_IRC1_PHY_ADDR		(PVC01S_IRC0_PHY_ADDR + PVC01S_IRC_PHY_SIZE)
#define PVC01S_IRC2_PHY_ADDR		(PVC01S_IRC1_PHY_ADDR + PVC01S_IRC_PHY_SIZE)


//	TIMER : target timer1
//	TIMER 0		addr : 0xfff07000
//				size : 0x1000(4096)
//	TIMER 1		addr : 0xfff08000
//				size : 0x1000(4096)
#define PVC01S_TIMER_PHY_SIZE		0x1000
#define PVC01S_TIMER0_PHY_ADDR		0xfff07000
#define PVC01S_TIMER1_PHY_ADDR		(PVC01S_TIMER0_PHY_ADDR + PVC01S_TIMER_PHY_SIZE)


//	UART : target =  1ch
//	UART0		addr : 0xfff20000
//				size : 0x1000(4096)
//	UART1		addr : 0xfff21000
//				size : 0x1000(4096)
#define PVC01S_UART_PHY_SIZE		0x1000
#define PVC01S_UART0_PHY_ADDR		0xfff20000
#define PVC01S_UART1_PHY_ADDR		(PVC01S_UART0_PHY_ADDR + PVC01S_UART_PHY_SIZE)


//	flash memory
#define PVC01S_CS_SIZE		0x04000000
#define PVC01S_CS0_PHY_ADDR	0x00000000
#define PVC01S_CS1_PHY_ADDR	(PVC01S_FLASH_CS0_PHY_ADDR+PVC01S_FLASH_CS0_SIZE)



//
//	logical	mapping
//
#define PVC01S_VIRT_REG_TOP			0xd0000000
#define PVC01S_VIRT_REG_MASK		0x0fffffff

//	irc group
#define PVC01S_IRC_GROUP_VIRT_ADDR		( PVC01S_VIRT_REG_TOP | ( PVC01S_IRCGROUP_PHY_ADDR & PVC01S_VIRT_REG_MASK))
//	irc1
#define PVC01S_IRC_TOP_MAP_VIRT_ADDR	( PVC01S_VIRT_REG_TOP | ( PVC01S_IRC_MAPPED_TOP_PHY_ADDR & PVC01S_VIRT_REG_MASK))
#define PVC01S_IRC_VIRT_ADDR			( PVC01S_IRC_TOP_MAP_VIRT_ADDR + (PVC01S_IRC1_PHY_ADDR - PVC01S_IRC_MAPPED_TOP_PHY_ADDR))
//	timer
#define PVC01S_TIMER_VIRT_ADDR			( PVC01S_VIRT_REG_TOP | ( PVC01S_TIMER1_PHY_ADDR & PVC01S_VIRT_REG_MASK))



//
//	hircgregister offset
//
#define HIRCG_CPU1_IRQSTATUS_G04_OFFSET		0x120
#define HIRCG_CPU1_IRQENABLE_G04_OFFSET		0x124

//	[0]timer_0-1
//	[1]timer_0-2
//	[2]timer_1-1
//	[3]timer_1-2
#define HIRCG_CPU1_IRQSTATUS_G19_OFFSET		0x198
//	enabler
#define HIRCG_CPU1_IRQENABLE_G19_OFFSET		0x19c

//	[0]timer_8-1
//	[1]timer_8-2
//	[2]timer_9-1
//	[3]timer_9-2
//       |
//	[8]timer_12-1
//	[9]timer_12-2
#define HIRCG_CPU1_IRQSTATUS_G21_OFFSET		0x1a8
//	enabler
#define HIRCG_CPU1_IRQENABLE_G21_OFFSET		0x1ac

//	[0]uart1 uart_intr stat
#define HIRCG_CPU1_IRQSTATUS_G30_OFFSET		0x1f0
//	[0]uart1 uart_intr ena/dis
#define HIRCG_CPU1_IRQENABLE_G30_OFFSET		0x1f4




//	interrupt register
#define IRQF						0x00
#define IRQM						0x04
#define ILM							0x08
#define ICRMN						0x0c
#define HRCL						0x10
#define SWIR0						0x14
#define SWIR1						0x18
#define TBR							0x1c
#define VCT							0x20
#define IRQTEST						0x24
#define FIQTEST						0x28
#define ICR00						0x30
#define ICR01                       0x34
#define ICR02                       0x38
#define ICR03                       0x3c
#define ICR04                       0x40
#define ICR05                       0x44
#define ICR06                       0x48
#define ICR07                       0x4c
#define ICR08                       0x50
#define ICR09                       0x54
#define ICR10                       0x58
#define ICR11                       0x5c
#define ICR12                       0x60
#define ICR13                       0x64
#define ICR14                       0x68
#define ICR15                       0x6c
#define ICR16                       0x70
#define ICR17                       0x74
#define ICR18                       0x78
#define ICR19                       0x7c
#define ICR20                       0x80
#define ICR21                       0x84
#define ICR22                       0x88
#define ICR23                       0x8c
#define ICR24                       0x90
#define ICR25                       0x94
#define ICR26                       0x98
#define ICR27                       0x9c
#define ICR28                       0xa0
#define ICR29                       0xa4
#define ICR30						0xa8
#define ICR31						0xac


//	timer define
#define MAX_TIMER                   2
#define MAX_PERIOD                  699050
#define TIMER_RELOAD				(CLOCK_TICK_RATE / HZ)
#define mSEC_1                      1000
#define mSEC_5                      (mSEC_1 * 5)
#define mSEC_10                     (mSEC_1 * 10)
#define mSEC_25                     (mSEC_1 * 25)
#define SEC_1                       (mSEC_1 * 1000)



#endif  /* __ASM_ARCH_PLATFORM_H */

