/*
 * linux/arch/arm/mach-pvc01s/irq.c
 *
 * Copyright (c) 2012 Panasonic, Inc
 *
 * based on : Jade SoC arch 
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/mm.h>
#include <linux/types.h>

#include <asm/irq.h>
#include <asm/mach-types.h>
#include <asm/setup.h>

#include <asm/mach/arch.h>
#include <asm/mach/irq.h>
#include <asm/mach/map.h>
#include <mach/pvc01s.h>


static int sdio_irq_mask = 0;
#if defined(PVCSD_USE_SD0)
#define PVC01S_IRQ_SIDIO_MASK (0x03)
#elif defined(PVCSD_USE_SD1)
#define PVC01S_IRQ_SIDIO_MASK (0x0C)
#elif defined(PVCSD_USE_SD2)
#define PVC01S_IRQ_SIDIO_MASK (0x30)
#endif

//
//	group register mask
//
static void pvc01s_group_ack(unsigned int irq)
{
	if ( irq == IRQ_SDIO){
		//	ack sdio  (group 4/bit[0]-bit[5])
		__raw_writel(PVC01S_IRQ_SIDIO_MASK, PVC01S_IRC_GROUP_VIRT_ADDR + HIRCG_CPU1_IRQSTATUS_G04_OFFSET);
	}else
	if ( irq == IRQ_TIMERINT1_1){
		//	ack timer
		__raw_writel(0x4, PVC01S_IRC_GROUP_VIRT_ADDR + HIRCG_CPU1_IRQSTATUS_G19_OFFSET);
	}else
	if ( irq == IRQ_UARTINT1){
		//	ack uart (group 19/bit[2])
		__raw_writel(0x1, PVC01S_IRC_GROUP_VIRT_ADDR + HIRCG_CPU1_IRQSTATUS_G30_OFFSET);
	}else{
#if 0
		//	check unknown irq
		printk("pvc01s_group_ack check = %d\n", irq );
#endif
	}
}

//	irc mask
static void pvc01s_mask_irq(unsigned int irq)
{
	//	mask
	__raw_writel(15, PVC01S_IRC_VIRT_ADDR + ICR00 + (irq << 2));
	//	clear 
	__raw_writel( 0, PVC01S_IRC_VIRT_ADDR + IRQF);
	//	dummy read
	__raw_readl (    PVC01S_IRC_VIRT_ADDR + IRQF);
}

//	irc unmask
static void pvc01s_unmask_irq(unsigned int irq)
{
	//	ack
	pvc01s_group_ack(irq);

	//	enable irc
	__raw_writel(13, PVC01S_IRC_VIRT_ADDR + ICR00 + ((irq) << 2));
}

static struct irq_chip pvc01s_ic_chip = {
	.name		= "PVC01S-IC",
	.ack		= pvc01s_mask_irq,
	.mask		= pvc01s_mask_irq,
	.unmask		= pvc01s_unmask_irq,
};

void __init pvc01s_init_interrupts(void)
{
	unsigned int i;

	for (i = 0; (i < NR_IRQS); i++) {
		set_irq_chip(i, &pvc01s_ic_chip);
		pvc01s_mask_irq(i);
		set_irq_handler(i, handle_level_irq);
		set_irq_flags(i, IRQF_VALID | IRQF_PROBE);
	}
	//	clear all interrupts
	__raw_writel( 0, PVC01S_IRC_VIRT_ADDR + IRQF);

	//	enable all interrupts
	__raw_writel( 1, PVC01S_IRC_VIRT_ADDR + IRQM);

	//	set interrupt level mask register
	__raw_writel(0xf, PVC01S_IRC_VIRT_ADDR + ILM);


	//
	//	group enable register
	//
	//	sdio -> irc
	__raw_writel(PVC01S_IRQ_SIDIO_MASK, PVC01S_IRC_GROUP_VIRT_ADDR + HIRCG_CPU1_IRQENABLE_G04_OFFSET);

	//	primecell uart -> irc
	__raw_writel(0x1, PVC01S_IRC_GROUP_VIRT_ADDR + HIRCG_CPU1_IRQENABLE_G30_OFFSET);

	//	timer ip -> irc
	__raw_writel(0x4, PVC01S_IRC_GROUP_VIRT_ADDR + HIRCG_CPU1_IRQENABLE_G19_OFFSET);
}
