/*
 * linux/arch/arm/mach-pvc01s/timer.c
 *
 * Copyright (c) 2012 Panasonic, Inc
 *
 * based on : arm-versatile/timer.c 
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/mm.h>
#include <linux/types.h>

#include <asm/irq.h>
#include <asm/mach-types.h>
#include <asm/setup.h>

#include <asm/mach/arch.h>
#include <asm/mach/irq.h>
#include <asm/mach/map.h>

#include <linux/init.h>
#include <linux/device.h>
#include <linux/dma-mapping.h>
#include <linux/platform_device.h>
#include <linux/sysdev.h>
#include <linux/interrupt.h>
#include <linux/amba/bus.h>
#include <linux/amba/mmci.h>
#include <linux/clocksource.h>
#include <linux/clockchips.h>
#include <linux/cnt32_to_63.h>
#include <linux/io.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/plat-ram.h>

#include <asm/clkdev.h>
#include <asm/system.h>
#include <mach/hardware.h>
#include <asm/irq.h>
#include <asm/hardware/arm_timer.h>
#include <asm/mach-types.h>

#include <asm/mach/irq.h>
#include <asm/mach/time.h>

#include "core.h"
#include "clock.h"

#ifdef CONFIG_GENERIC_CLOCKEVENTS
static void timer_set_mode(enum clock_event_mode mode,
			   struct clock_event_device *clk)
{
	unsigned long ctrl;

	switch(mode) {
	case CLOCK_EVT_MODE_PERIODIC:
		__raw_writel(TIMER_RELOAD, PVC01S_TIMER_VIRT_ADDR + TIMER_LOAD);
		ctrl = TIMER_CTRL_PERIODIC;
		ctrl |= TIMER_CTRL_32BIT | TIMER_CTRL_IE | TIMER_CTRL_ENABLE;
		break;
	case CLOCK_EVT_MODE_ONESHOT:
		/* period set, and timer enabled in 'next_event' hook */
		ctrl = TIMER_CTRL_ONESHOT;
		ctrl |= TIMER_CTRL_32BIT | TIMER_CTRL_IE;
		break;
	case CLOCK_EVT_MODE_UNUSED:
	case CLOCK_EVT_MODE_SHUTDOWN:
	default:
		ctrl = 0;
	}

	__raw_writel(ctrl, PVC01S_TIMER_VIRT_ADDR + TIMER_CTRL);
}

static int timer_set_next_event(unsigned long evt,
				struct clock_event_device *unused)
{
	unsigned long ctrl = readl(PVC01S_TIMER_VIRT_ADDR + TIMER_CTRL);

	__raw_writel(evt, PVC01S_TIMER_VIRT_ADDR + TIMER_LOAD);
	__raw_writel(ctrl | TIMER_CTRL_ENABLE, PVC01S_TIMER_VIRT_ADDR + TIMER_CTRL);

	return 0;
}

static struct clock_event_device timer0_clockevent =	 {
	.name		= "timer0",
	.shift		= 32,
	.features       = CLOCK_EVT_FEAT_PERIODIC | CLOCK_EVT_FEAT_ONESHOT,
	.set_mode	= timer_set_mode,
	.set_next_event	= timer_set_next_event,
};

//	pvc01s system timer
static irqreturn_t pvc01s_timer_interrupt(int irq, void *dev_id)
{
	struct clock_event_device *evt = &timer0_clockevent;

	//	ack timer ip
	__raw_writel(1, PVC01S_TIMER_VIRT_ADDR + TIMER_INTCLR);

	evt->event_handler(evt);

	return IRQ_HANDLED;
}

static struct irqaction pvc01s_timer_irq = {
	.name		= "Timer Tick",
	.flags		= IRQF_DISABLED | IRQF_TIMER | IRQF_IRQPOLL,
	.handler	= pvc01s_timer_interrupt,
};


static void __init pvc01s_timer_init(void)
{
	//	timer reset
	__raw_writel(0, PVC01S_TIMER_VIRT_ADDR + TIMER_CTRL);

	//	register irq
	setup_irq(IRQ_TIMERINT1_1, &pvc01s_timer_irq);

	timer0_clockevent.mult =
		div_sc(CLOCK_TICK_RATE, NSEC_PER_SEC, timer0_clockevent.shift);
	timer0_clockevent.max_delta_ns =
		clockevent_delta2ns(0xffffffff, &timer0_clockevent);
	timer0_clockevent.min_delta_ns =
		clockevent_delta2ns(0xf, &timer0_clockevent);

	timer0_clockevent.cpumask = cpumask_of(0);
	clockevents_register_device(&timer0_clockevent);
}

#else
//	for timer check
static irqreturn_t pvc01s_timer_interrupt(int irq, void *dev_id)
{
	//	ack timer ip
	__raw_writel(1, PVC01S_TIMER_VIRT_ADDR + TIMER_INTCLR);
	timer_tick();

	return IRQ_HANDLED;
}

static struct irqaction pvc01s_timer_irq = {
	.name		= "Timer Tick",
	.flags		= IRQF_DISABLED | IRQF_TIMER,
	.handler	= pvc01s_timer_interrupt,
};

static void __init pvc01s_timer_init(void)
{
	unsigned long ctrl;

	//	timer reset
	__raw_writel(0, PVC01S_TIMER_VIRT_ADDR + TIMER_CTRL);

	//	register irq
	setup_irq(IRQ_TIMERINT1_1, &pvc01s_timer_irq);

	__raw_writel(TIMER_RELOAD, PVC01S_TIMER_VIRT_ADDR + TIMER_LOAD);
	ctrl = TIMER_CTRL_PERIODIC;
	ctrl |= TIMER_CTRL_32BIT | TIMER_CTRL_IE | TIMER_CTRL_ENABLE;
	__raw_writel(ctrl, PVC01S_TIMER_VIRT_ADDR + TIMER_CTRL);
}
#endif

struct sys_timer pvc01s_timer = {
	.init		= pvc01s_timer_init,
};
