/*
 * Panasonic pvc02v board Support
 */

#include <linux/init.h>
#include <linux/gfp.h>
#include <linux/device.h>
#include <linux/platform_device.h>
#include <linux/amba/bus.h>
#include <linux/irq.h>
#include <linux/io.h>
#include <linux/init.h>
#include <linux/spinlock.h>

#include <asm/clkdev.h>
#include <asm/mach-types.h>
#include <asm/sizes.h>
#include <asm/smp_plat.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/time.h>
#include <asm/hardware/arm_timer.h>
#include <asm/hardware/gic.h>
#include <asm/hardware/sp810.h>

#include <mach/clkdev.h>
#include <mach/irqs.h>

#include <mach/board.h>

#ifdef CONFIG_PVC_CMN_EXCLUSION
#include <common_spin_lock.h>
#endif

#include "core.h"
#include "timer-sp.h"

#if defined(CONFIG_ARCH_PVC02V)
#include <mach/pvc01v.h>
#endif

static struct map_desc pvc_io_desc[] __initdata = {
	{
		.virtual        = PVC_GIC_BASE,
		.pfn            = __phys_to_pfn(PVC_GIC_START),
		.length         = PVC_GIC_SIZE,
		.type           = MT_DEVICE,
	},
	{
		.virtual        = PVC_UART_BASE,
		.pfn            = __phys_to_pfn(PVC_UART_START),
		.length         = PVC_UART_SIZE,
		.type           = MT_DEVICE,
	},
	{
		.virtual        = PVC_TIMER_BASE,
		.pfn            = __phys_to_pfn(PVC_TIMER_START),
		.length         = PVC_TIMER_SIZE,
		.type           = MT_DEVICE,
	},
	{
		.virtual        = PVC_CKR_BASE,
		.pfn            = __phys_to_pfn(PVC_CKR_START),
		.length         = PVC_CKR_SIZE,
		.type           = MT_DEVICE,
	},
	{
		.virtual	= PVC_KERNEL_UNCACHE_BASE,
		.pfn		= __phys_to_pfn(PVC_KERNEL_UNCACHE_START),
		.length		= PVC_KERNEL_UNCACHE_SIZE,
		.type		= MT_DEVICE,
	},
#ifdef CONFIG_PVC_CMN_EXCLUSION
	{
		.virtual	= LINUX_EXCLUSION_MAP_VADDR,
		.pfn		= __phys_to_pfn(LINUX_EXCLUSION_MAP_PADDR),
		.length		= LINUX_EXCLUSION_MAP_SIZE,
		.type		= MT_MEMORY_PVC_EXCLUSION,
	},
#endif
};

void __iomem *gic_cpu_base_addr  = (void*)PVC_GIC_CPU_VADDR;
void __iomem *gic_dist_base_addr = (void*)PVC_GIC_DIST_VADDR;

extern void gic_set_percpu_offset(unsigned int gic_nr, unsigned int percpu_offset);
extern void gic_dist_init_selfcpu(unsigned int gic_nr, void __iomem *base,
				  unsigned int irq_start, u8 cpumask);
extern void gic_set_irq_chip(unsigned int gic_nr, unsigned int irq);
extern void set_irq_flags(unsigned int irq, unsigned int iflags);

static void __init pvc_init_irq(void)
{
#ifdef CONFIG_PVC_GIC_DIST_INIT
	u8 cpumask = 0x00;

#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU0
	cpumask |= (1 << 0);
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU1
	cpumask |= (1 << 1);
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU2
	cpumask |= (1 << 2);
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU3
	cpumask |= (1 << 3);
#endif
	gic_set_percpu_offset(0, PVC_GIC_CPU_OFFSET);
	gic_dist_init_selfcpu(0, gic_dist_base_addr, 32, cpumask);
#else /* !CONFIG_PVC_GIC_DIST_INIT */
	gic_set_percpu_offset(0, PVC_GIC_CPU_OFFSET);
	gic_dist_init(0, gic_dist_base_addr, 32);
#endif

#ifdef PVC_REQUEST_IRQ_SGI
	{
		int i;
		i = PVC_REQUEST_IRQ_SGI2;
		gic_set_irq_chip(0, i);
		set_irq_handler(i, handle_level_irq);
		set_irq_flags(i, IRQF_VALID | IRQF_PROBE);

		i = PVC_REQUEST_IRQ_SGI6;
		gic_set_irq_chip(0, i);
		set_irq_handler(i, handle_level_irq);
		set_irq_flags(i, IRQF_VALID | IRQF_PROBE);
	}
#endif
	gic_cpu_init(0, gic_cpu_base_addr);
}


#ifdef CONFIG_SMP

extern void pvc_secondary_startup(void);


void __init board_smp_init_cpus(void)
{
}

void __init board_smp_prepare_cpus(unsigned int max_cpus)
{
	/*
	 * Write the address of secondary startup into
	 * the synchronous flag. The boot monitor waits
	 * until it receives a soft interrupt, and then the
	 * secondary CPU branches to this address.
	 */
	writel(virt_to_phys(pvc_secondary_startup), PVC_SMP_SYNC_VADDR);
}

#endif

#ifdef CONFIG_PVC_CPU_LOGICAL_MAP

static const int __mpidr_list[] = {
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU0
	0x000,
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU1
	0x001,
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU2
	0x002,
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU3
	0x003,
#endif
};

static const int __gic_cpuif_list[] = {
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU0
	0,
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU1
	1,
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU2
	2,
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU3
	3,
#endif
};

#define MPIDR_LIST_NR  (sizeof(__mpidr_list) / sizeof(int))

void __init cpu_logical_map_init(void)
{
	int i;
	/* Initialize logical_map */
	for (i = 0; i < MPIDR_LIST_NR; ++i) {
		/* CPU map for MPIDR ID */
		cpu_logical_map(i) = __mpidr_list[i];
		/* CPU map for GIC CPU Interface */
		cpuif_logical_map(i) = __gic_cpuif_list[i];

		set_cpu_possible(i, true);
	}
}

/* Check configuration */
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU0
#define _CA7_CPU0	1
#else
#define _CA7_CPU0	0
#endif

#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU1
#define _CA7_CPU1	1
#else
#define _CA7_CPU1	0
#endif

#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU2
#define _CA7_CPU2	1
#else
#define _CA7_CPU2	0
#endif

#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU3
#define _CA7_CPU3	1
#else
#define _CA7_CPU3	0
#endif

#if CONFIG_NR_CPUS < (_CA7_CPU0 + _CA7_CPU1 + _CA7_CPU2 + _CA7_CPU3)
#error Maximum number of CPUs is not correct.
#endif

#endif /* CONFIG_PVC_CPU_LOGICAL_MAP */

static AMBA_APB_DEVICE(uart0, "mb:uart0", 0, PVC_UART0_PADDR, IRQ_PVC_UART0, NULL);
static AMBA_APB_DEVICE(uart1, "mb:uart1", 0, PVC_UART1_PADDR, IRQ_PVC_UART1, NULL);
static AMBA_APB_DEVICE(uart2, "mb:uart2", 0, PVC_UART2_PADDR, IRQ_PVC_UART2, NULL);
static AMBA_APB_DEVICE(uart3, "mb:uart3", 0, PVC_UART3_PADDR, IRQ_PVC_UART3, NULL);

static struct amba_device *pvc_amba_devs[] __initdata = {
	&uart0_device,
	&uart1_device,
	&uart2_device,
	&uart3_device,
};

static struct clk apb_clk = {
	.rate	= PVC_PERI_CLK,
};

static struct clk_lookup lookups[] = {
	{
		.dev_id		= "mb:uart0",
		.clk		= &apb_clk,
  	}, {
		.dev_id		= "mb:uart1",
		.clk		= &apb_clk,
  	}, {
		.dev_id		= "mb:uart2",
		.clk		= &apb_clk,
  	}, {
		.dev_id		= "mb:uart3",
		.clk		= &apb_clk,
	}
};

static void __init pvc_clk_init(void)
{
	int i;
	for (i = 0; i < ARRAY_SIZE(lookups); i++) {
		clkdev_add(&lookups[i]);
	}
}

static void __init pvc_timer_init(void)
{
	void __iomem *base;
	u32 value;

	/* reset cancel */
	value = (PVC_HCKR_TIMER_20_21_PRESET | PVC_HCKR_TIMER_22_23_PRESET);
	writel(value, PVC_HCKR_RST12_VADDR);

	/* clock on */
	value = (PVC_HCKR_TIMER_20_21_PCLK | PVC_HCKR_TIMER_20_21_TIMCLK |
		 PVC_HCKR_TIMER_22_23_PCLK | PVC_HCKR_TIMER_22_23_TIMCLK);
	writel(value, PVC_HCKR_CLK12_VADDR);

	pvc_clk_init();

	base = (void *)PVC_TIMER_22_23_VADDR;

	writel(0, base + TIMER_1_BASE + TIMER_CTRL);
	writel(0, base + TIMER_2_BASE + TIMER_CTRL);

	sp804_clocksource_init(base + TIMER_1_BASE);
	sp804_clockevents_init(base + TIMER_2_BASE, IRQ_PVC_TIMER23);
}

static struct sys_timer pvc_timer = {
	.init	= pvc_timer_init,
};

static void pvc_power_off(void)
{

}

void arch_reset(char mode, const char *cmd)
{

}

static void __init pvc_map_io(void)
{
	iotable_init(pvc_io_desc, ARRAY_SIZE(pvc_io_desc));
}

#if defined(CONFIG_ARCH_PVC02V)
static struct resource pvcsdio_resources[] = {
	/* [0] */
	{
		.start	= PVCSD_SDX_BASE_CH0,
		.end	= PVCSD_SDX_BASE_CH0 + SZ_4K - 1,
		.flags	= IORESOURCE_MEM,
	},
	/* [1] */
	{
		.start	= PVCSD_SDX_BASE_CH1,
		.end	= PVCSD_SDX_BASE_CH1 + SZ_4K - 1,
		.flags	= IORESOURCE_MEM,
	},
	/* [2] */
	{
		.start	= PVCSD_SDX_BASE_CH2,
		.end	= PVCSD_SDX_BASE_CH2 + SZ_4K - 1,
		.flags	= IORESOURCE_MEM,
	},
	/* [3] */
	{
		.start	= PVCSD_SDX_BASE_CH3,
		.end	= PVCSD_SDX_BASE_CH3 + SZ_4K - 1,
		.flags	= IORESOURCE_MEM,
	},
	/* [4] */
	{
		.start	= IRQ_PVC_SDC0_SDIP,
		.end	= IRQ_PVC_SDC0_SDIP,
		.flags	= IORESOURCE_IRQ,
	},
	/* [5] */
	{
		.start	= IRQ_PVC_SDC0_SDIP2,
		.end	= IRQ_PVC_SDC0_SDIP2,
		.flags	= IORESOURCE_IRQ,
	},
	/* [6] */
	{
		.start	= IRQ_PVC_SDC1_SDIP,
		.end	= IRQ_PVC_SDC1_SDIP,
		.flags	= IORESOURCE_IRQ,
	},
	/* [7] */
	{
		.start	= IRQ_PVC_SDC1_SDIP2,
		.end	= IRQ_PVC_SDC1_SDIP2,
		.flags	= IORESOURCE_IRQ,
	},
	/* [8] */
	{
		.start	= IRQ_PVC_SDC2_SDIP,
		.end	= IRQ_PVC_SDC2_SDIP,
		.flags	= IORESOURCE_IRQ,
	},
	/* [9] */
	{
		.start	= IRQ_PVC_SDC2_SDIP2,
		.end	= IRQ_PVC_SDC2_SDIP2,
		.flags	= IORESOURCE_IRQ,
	},
	/* [10] */
	{
		.start	= IRQ_PVC_SDC3_SDIP,
		.end	= IRQ_PVC_SDC3_SDIP,
		.flags	= IORESOURCE_IRQ,
	},
	/* [11] */
	{
		.start	= IRQ_PVC_SDC3_SDIP2,
		.end	= IRQ_PVC_SDC3_SDIP2,
		.flags	= IORESOURCE_IRQ,
	},
};

static u64 pvcsdio_dmamask = 0xffffffffUL;

static struct platform_device pvc01v_sdio = {
	.name		= "pvsdio",
	.id		= -1,
	.dev		= {
		.dma_mask = &pvcsdio_dmamask,
		.coherent_dma_mask = 0xffffffff,
	},
	.num_resources	= ARRAY_SIZE(pvcsdio_resources),
	.resource	= pvcsdio_resources,
};

static struct pvcsdio_platform_data pvcsdio_data;
#endif /* CONFIG_ARCH_PVC02V */

static void __init pvc_init(void)
{
	int i;

	for (i = 0; i < ARRAY_SIZE(pvc_amba_devs); i++)
		amba_device_register(pvc_amba_devs[i], &iomem_resource);

	pm_power_off = pvc_power_off;

#if defined(CONFIG_ARCH_PVC02V)
#if defined(CONFIG_MMC_PVC02V) || defined(CONFIG_MMC_PVC02V_MODULE)
	pvcsdio_data.clock = 43875000;
	pvc01v_sdio.dev.platform_data = &pvcsdio_data;
	platform_device_register(&pvc01v_sdio);
#endif
#endif
}

MACHINE_START(PVC02V, "Panasonic pvc02v board")
	.map_io		= pvc_map_io,
	.init_irq	= pvc_init_irq,
	.timer		= &pvc_timer,
	.init_machine	= pvc_init,
	.phys_io	= PVC02V_DEBUG_UART_PADDR,
	.io_pg_offst	= (PVC02V_DEBUG_UART_VADDR >> 18) & 0xfffc,
MACHINE_END
