/*
 * Versatile Express Core Tile Cortex A15_a7 Support
 */

#include <linux/init.h>
#include <linux/gfp.h>
#include <linux/device.h>
#include <linux/platform_device.h>
#include <linux/amba/bus.h>
#include <linux/irq.h>
#include <linux/io.h>
#include <linux/init.h>
#include <linux/spinlock.h>

#include <asm/clkdev.h>
#include <asm/mach-types.h>
#include <asm/sizes.h>
#include <asm/smp_plat.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/time.h>
#include <asm/hardware/arm_timer.h>
#include <asm/hardware/gic.h>
#include <asm/hardware/sp810.h>

#include <mach/clkdev.h>
#include <mach/irqs.h>

#include <mach/board.h>

#ifdef CONFIG_PVC_CMN_EXCLUSION
#include <common_spin_lock.h>
#endif

#include "core.h"
#include "timer-sp.h"

static struct map_desc v2m_io_desc[] __initdata = {
	{
		.virtual        = V2M_GIC_BASE,
		.pfn            = __phys_to_pfn(V2M_GIC_START),
		.length         = V2M_GIC_SIZE,
		.type           = MT_DEVICE,
	},
	{
		.virtual	= V2M_PERI_BASE,
		.pfn		= __phys_to_pfn(V2M_PERI_START),
		.length		= V2M_PERI_SIZE,
		.type		= MT_DEVICE,
	},
	{
		.virtual	= PVC_KERNEL_UNCACHE_BASE,
		.pfn		= __phys_to_pfn(PVC_KERNEL_UNCACHE_START),
		.length		= PVC_KERNEL_UNCACHE_SIZE,
		.type		= MT_DEVICE,
	},
#ifdef CONFIG_PVC_CMN_EXCLUSION
	{
		.virtual	= LINUX_EXCLUSION_MAP_VADDR,
		.pfn		= __phys_to_pfn(LINUX_EXCLUSION_MAP_PADDR),
		.length		= LINUX_EXCLUSION_MAP_SIZE,
		.type		= MT_MEMORY_PVC_EXCLUSION,
	},
#endif
};


void __iomem *gic_cpu_base_addr  = (void*)V2M_GIC_CPU_VADDR;
void __iomem *gic_dist_base_addr = (void*)V2M_GIC_DIST_VADDR;

extern void gic_set_percpu_offset(unsigned int gic_nr, unsigned int percpu_offset);
extern void gic_dist_init_selfcpu(unsigned int gic_nr, void __iomem *base,
				  unsigned int irq_start, u8 cpumask);
extern void gic_set_irq_chip(unsigned int gic_nr, unsigned int irq);
extern void set_irq_flags(unsigned int irq, unsigned int iflags);

static void __init v2m_init_irq(void)
{
#ifdef CONFIG_PVC_GIC_DIST_INIT
	u8 cpumask = 0x00;

#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU0
	cpumask |= (1 << 2);
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU1
	cpumask |= (1 << 3);
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU2
	cpumask |= (1 << 4);
#endif
	gic_set_percpu_offset(0, V2M_GIC_CPU_OFFSET);
	gic_dist_init_selfcpu(0, gic_dist_base_addr, 32, cpumask);
#else /* !CONFIG_PVC_GIC_DIST_INIT */
	gic_set_percpu_offset(0, V2M_GIC_CPU_OFFSET);
	gic_dist_init(0, gic_dist_base_addr, 32);
#endif /* !CONFIG_PVC_GIC_DIST_INIT */

#ifdef PVC_REQUEST_IRQ_SGI
	{
		int i;
		i = PVC_REQUEST_IRQ_SGI2;
		gic_set_irq_chip(0, i);
		set_irq_handler(i, handle_level_irq);
		set_irq_flags(i, IRQF_VALID | IRQF_PROBE);

		i = PVC_REQUEST_IRQ_SGI6;
		gic_set_irq_chip(0, i);
		set_irq_handler(i, handle_level_irq);
		set_irq_flags(i, IRQF_VALID | IRQF_PROBE);

	}
#endif
	gic_cpu_init(0, gic_cpu_base_addr);
}



#ifdef CONFIG_SMP

extern void pvc_secondary_startup(void);


void __init board_smp_init_cpus(void)
{
}

void __init board_smp_prepare_cpus(unsigned int max_cpus)
{
	/*
	 * Write the address of secondary startup into
	 * the synchronous flag. The boot monitor waits
	 * until it receives a soft interrupt, and then the
	 * secondary CPU branches to this address.
	 */
	writel(virt_to_phys(pvc_secondary_startup), PVC_SMP_SYNC_VADDR);
}

#endif

#ifdef CONFIG_PVC_CPU_LOGICAL_MAP

static const int __mpidr_list[] = {
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU0
	0x100,
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU1
	0x101,
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU2
	0x102,
#endif
};

static const int __gic_cpuif_list[] = {
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU0
	2,
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU1
	3,
#endif
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU2
	4,
#endif
};

#define MPIDR_LIST_NR	(sizeof(__mpidr_list) / sizeof(int))

void __init cpu_logical_map_init(void)
{
	int i;
	/* Initialize logical_map */
	for (i = 0; i < MPIDR_LIST_NR ; ++i) {
		/* CPU map for MPIDR ID */
		cpu_logical_map(i) = __mpidr_list[i];
		/* CPU map for GIC CPU Interface */
		cpuif_logical_map(i) = __gic_cpuif_list[i];

		set_cpu_possible(i, true);
	}
}

/* Check configuration */
#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU0
#define _CA7_CPU0	1
#else
#define _CA7_CPU0	0
#endif

#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU1
#define _CA7_CPU1	1
#else
#define _CA7_CPU1	0
#endif

#ifdef CONFIG_PVC_LINUX_ON_CA7_CPU2
#define _CA7_CPU2	1
#else
#define _CA7_CPU2	0
#endif

#if CONFIG_NR_CPUS < (_CA7_CPU0 + _CA7_CPU1 + _CA7_CPU2)
#error Maximum number of CPUs is not correct.
#endif

#endif /* CONFIG_PVC_CPU_LOGICAL_MAP */

static void __iomem *v2m_sysreg_base;

static void __init v2m_sysctl_init(void __iomem *base)
{
	u32 scctrl;

	if (WARN_ON(!base))
		return;

	/* Select 1MHz TIMCLK as the reference clock for SP804 timers */
	scctrl = readl(base + SCCTRL);
	scctrl |= SCCTRL_TIMEREN0SEL_TIMCLK;
	scctrl |= SCCTRL_TIMEREN1SEL_TIMCLK;
	writel(scctrl, base + SCCTRL);
}


static DEFINE_SPINLOCK(v2m_cfg_lock);

int v2m_cfg_write(u32 devfn, u32 data)
{
	/* Configuration interface broken? */
	u32 val;

	printk("%s: writing %08x to %08x\n", __func__, data, devfn);

	devfn |= SYS_CFG_START | SYS_CFG_WRITE;

	spin_lock(&v2m_cfg_lock);
	val = readl(v2m_sysreg_base + V2M_SYS_CFGSTAT);
	writel(val & ~SYS_CFG_COMPLETE, v2m_sysreg_base + V2M_SYS_CFGSTAT);

	writel(data, v2m_sysreg_base +  V2M_SYS_CFGDATA);
	writel(devfn, v2m_sysreg_base + V2M_SYS_CFGCTRL);

	do {
		val = readl(v2m_sysreg_base + V2M_SYS_CFGSTAT);
	} while (val == 0);
	spin_unlock(&v2m_cfg_lock);

	return !!(val & SYS_CFG_ERR);
}

int v2m_cfg_read(u32 devfn, u32 *data)
{
	u32 val;

	devfn |= SYS_CFG_START;

	spin_lock(&v2m_cfg_lock);
	writel(0, v2m_sysreg_base + V2M_SYS_CFGSTAT);
	writel(devfn, v2m_sysreg_base + V2M_SYS_CFGCTRL);

	mb();

	do {
		cpu_relax();
		val = readl(v2m_sysreg_base + V2M_SYS_CFGSTAT);
	} while (val == 0);

	*data = readl(v2m_sysreg_base + V2M_SYS_CFGDATA);
	spin_unlock(&v2m_cfg_lock);

	return !!(val & SYS_CFG_ERR);
}

static AMBA_APB_DEVICE(uart0, "mb:uart0", 0, V2M_UART0_PADDR, IRQ_V2M_UART0, NULL);
static AMBA_APB_DEVICE(uart1, "mb:uart1", 0, V2M_UART1_PADDR, IRQ_V2M_UART1, NULL);
static AMBA_APB_DEVICE(uart2, "mb:uart2", 0, V2M_UART2_PADDR, IRQ_V2M_UART2, NULL);
static AMBA_APB_DEVICE(uart3, "mb:uart3", 0, V2M_UART3_PADDR, IRQ_V2M_UART3, NULL);

static struct amba_device *v2m_amba_devs[] __initdata = {
	&uart0_device,
	&uart1_device,
	&uart2_device,
	&uart3_device,
};

static struct clk osc2_clk = {
	.rate	= PVC_PERI_CLK,
};

static struct clk_lookup lookups[] = {
	{
		.dev_id		= "mb:uart0",
		.clk		= &osc2_clk,
  	}, {
		.dev_id		= "mb:uart1",
		.clk		= &osc2_clk,
  	}, {
		.dev_id		= "mb:uart2",
		.clk		= &osc2_clk,
  	}, {
		.dev_id		= "mb:uart3",
		.clk		= &osc2_clk,
	}
};

static void __init v2m_clk_init(void)
{
	int i;

	for (i = 0; i < ARRAY_SIZE(lookups); i++) {
		clkdev_add(&lookups[i]);
	}
}

static void __init v2m_timer_init(void)
{
	void __iomem *base;

	v2m_sysctl_init((void *)V2M_PERI_VADDR(V2M_SYSCTL));

	v2m_clk_init();

	base = (void *)V2M_PERI_VADDR(V2M_TIMER_0_1_PADDR);

	writel(0, base + TIMER_1_BASE + TIMER_CTRL);
	writel(0, base + TIMER_2_BASE + TIMER_CTRL);

	sp804_clocksource_init(base + TIMER_2_BASE);
	sp804_clockevents_init(base + TIMER_1_BASE, IRQ_V2M_TIMER0);
}

static struct sys_timer v2m_timer = {
	.init	= v2m_timer_init,
};

static void v2m_power_off(void)
{
	if (v2m_cfg_write(SYS_CFG_SHUTDOWN | SYS_CFG_SITE_MB, 0))
		printk(KERN_EMERG "Unable to shutdown\n");
}

void arch_reset(char mode, const char *cmd)
{
	if (v2m_cfg_write(SYS_CFG_REBOOT | SYS_CFG_SITE_MB, 0))
		printk(KERN_EMERG "Unable to reboot\n");
}

static void __init v2m_map_io(void)
{
	iotable_init(v2m_io_desc, ARRAY_SIZE(v2m_io_desc));

	v2m_sysreg_base = (void *)V2M_PERI_VADDR(V2M_SYSREGS);
}

static void __init v2m_init(void)
{
	int i;

	for (i = 0; i < ARRAY_SIZE(v2m_amba_devs); i++)
		amba_device_register(v2m_amba_devs[i], &iomem_resource);

	pm_power_off = v2m_power_off;
}

MACHINE_START(PVC02V, "ARM-Versatile Express")
	.map_io		= v2m_map_io,
	.init_irq	= v2m_init_irq,
	.timer		= &v2m_timer,
	.init_machine	= v2m_init,
	.phys_io	= PVC02V_DEBUG_UART_PADDR,
	.io_pg_offst	= (PVC02V_DEBUG_UART_VADDR >> 18) & 0xfffc,
MACHINE_END
