#ifndef __MACH_PVC_H
#define __MACH_PVC_H

/* PVC02V peripherals static mapping */

#define PVC_GIC_BASE		0xfe000000	/* V.A. */
#define PVC_GIC_START		0x70000000	/* P.A. */
#define PVC_GIC_SIZE		0x00020000

#define PVC_UART_BASE		0xfe020000	/* V.A. */
#define PVC_UART_START		0x7f050000	/* P.A. */
#define PVC_UART_SIZE		0x00004000

#define PVC_CKR_BASE		0xfe028000	/* V.A. */
#define PVC_CKR_START		0x7f038000	/* P.A. */
#define PVC_CKR_SIZE		0x00001000

#define PVC_TIMER_BASE		0xfe030000	/* V.A. */
#define PVC_TIMER_START		0x7f020000	/* P.A. */
#define PVC_TIMER_SIZE		0x0000d000

#define PVC_KERNEL_UNCACHE_BASE		0xfe040000
#define PVC_KERNEL_UNCACHE_START	CONFIG_KERNEL_RAM_BASE_ADDRESS	/* P.A. */
#define PVC_KERNEL_UNCACHE_SIZE		0x00004000

#define PVC_GIC_CPU_PADDR	(PVC_GIC_START + 0x00002000)
#define PVC_GIC_CPU_VADDR	(PVC_GIC_BASE  + 0x00002000)
#define PVC_GIC_DIST_PADDR	(PVC_GIC_START + 0x00001000)
#define PVC_GIC_DIST_VADDR	(PVC_GIC_BASE  + 0x00001000)
#define PVC_GIC_CPU_OFFSET	0x00008000

#define PVC_UART0_PADDR		(PVC_UART_START + 0x00000000)
#define PVC_UART0_VADDR		(PVC_UART_BASE  + 0x00000000)
#define PVC_UART1_PADDR		(PVC_UART_START + 0x00001000)
#define PVC_UART1_VADDR		(PVC_UART_BASE  + 0x00001000)
#define PVC_UART2_PADDR		(PVC_UART_START + 0x00002000)
#define PVC_UART2_VADDR		(PVC_UART_BASE  + 0x00002000)
#define PVC_UART3_PADDR		(PVC_UART_START + 0x00003000)
#define PVC_UART3_VADDR		(PVC_UART_BASE  + 0x00003000)

#define PVC_TIMER_0_1_PADDR	(PVC_TIMER_START + 0x00000000)
#define PVC_TIMER_0_1_VADDR	(PVC_TIMER_BASE  + 0x00000000)
#define PVC_TIMER_2_3_PADDR	(PVC_TIMER_START + 0x00001000)
#define PVC_TIMER_2_3_VADDR	(PVC_TIMER_BASE  + 0x00001000)
#define PVC_TIMER_4_5_PADDR	(PVC_TIMER_START + 0x00002000)
#define PVC_TIMER_4_5_VADDR	(PVC_TIMER_BASE  + 0x00002000)
#define PVC_TIMER_6_7_PADDR	(PVC_TIMER_START + 0x00003000)
#define PVC_TIMER_6_7_VADDR	(PVC_TIMER_BASE  + 0x00003000)
#define PVC_TIMER_8_9_PADDR	(PVC_TIMER_START + 0x00004000)
#define PVC_TIMER_8_9_VADDR	(PVC_TIMER_BASE  + 0x00004000)
#define PVC_TIMER_10_11_PADDR	(PVC_TIMER_START + 0x00005000)
#define PVC_TIMER_10_11_VADDR	(PVC_TIMER_BASE  + 0x00005000)
#define PVC_TIMER_12_13_PADDR	(PVC_TIMER_START + 0x00006000)
#define PVC_TIMER_12_13_VADDR	(PVC_TIMER_BASE  + 0x00006000)
#define PVC_TIMER_14_15_PADDR	(PVC_TIMER_START + 0x00007000)
#define PVC_TIMER_14_15_VADDR	(PVC_TIMER_BASE  + 0x00007000)
#define PVC_TIMER_16_17_PADDR	(PVC_TIMER_START + 0x00008000)
#define PVC_TIMER_16_17_VADDR	(PVC_TIMER_BASE  + 0x00008000)
#define PVC_TIMER_18_19_PADDR	(PVC_TIMER_START + 0x00009000)
#define PVC_TIMER_18_19_VADDR	(PVC_TIMER_BASE  + 0x00009000)
#define PVC_TIMER_20_21_PADDR	(PVC_TIMER_START + 0x0000a000)
#define PVC_TIMER_20_21_VADDR	(PVC_TIMER_BASE  + 0x0000a000)
#define PVC_TIMER_22_23_PADDR	(PVC_TIMER_START + 0x0000b000)
#define PVC_TIMER_22_23_VADDR	(PVC_TIMER_BASE  + 0x0000b000)
#define PVC_TIMER_24_25_PADDR	(PVC_TIMER_START + 0x0000c000)
#define PVC_TIMER_24_25_VADDR	(PVC_TIMER_BASE  + 0x0000c000)

/* Timer10(20_21), Timer11(22_23) clock */
#define PVC_HCKR_CLK12_PADDR		(PVC_CKR_START + 0x00000060)
#define PVC_HCKR_CLK12_VADDR		(PVC_CKR_BASE + 0x00000060)
#define PVC_HCKR_TIMER_20_21_PCLK	0x00000001
#define PVC_HCKR_TIMER_20_21_TIMCLK	0x00000002
#define PVC_HCKR_TIMER_22_23_PCLK	0x00000004
#define PVC_HCKR_TIMER_22_23_TIMCLK	0x00000008

/* Timer10(20_21), Timer11(22_23) reset */
#define PVC_HCKR_RST12_PADDR		(PVC_CKR_START + 0x00000064)
#define PVC_HCKR_RST12_VADDR		(PVC_CKR_BASE + 0x00000064)
#define PVC_HCKR_TIMER_20_21_PRESET	0x00000001
#define PVC_HCKR_TIMER_22_23_PRESET	0x00000002

/* UART definitions for early kernel messages */
#if defined(CONFIG_PVC_EARLY_UART0)
#define PVC02V_DEBUG_UART_PADDR	PVC_UART0_PADDR
#define PVC02V_DEBUG_UART_VADDR	PVC_UART0_VADDR
#elif defined(CONFIG_PVC_EARLY_UART1)
#define PVC02V_DEBUG_UART_PADDR	PVC_UART1_PADDR
#define PVC02V_DEBUG_UART_VADDR	PVC_UART1_VADDR
#elif defined(CONFIG_PVC_EARLY_UART2)
#define PVC02V_DEBUG_UART_PADDR	PVC_UART2_PADDR
#define PVC02V_DEBUG_UART_VADDR	PVC_UART2_VADDR
#elif defined(CONFIG_PVC_EARLY_UART3)
#define PVC02V_DEBUG_UART_PADDR	PVC_UART3_PADDR
#define PVC02V_DEBUG_UART_VADDR	PVC_UART3_VADDR
#else /* default */
#define PVC02V_DEBUG_UART_PADDR	PVC_UART0_PADDR
#define PVC02V_DEBUG_UART_VADDR	PVC_UART0_VADDR
#endif

/* SMP synchronous flag */
#define PVC_SMP_SYNC_PADDR	(PVC_KERNEL_UNCACHE_START + 0x3000)
#define PVC_SMP_SYNC_VADDR	(PVC_KERNEL_UNCACHE_BASE  + 0x3000)

/*
 * Interrupts.  Those in {} are for AMBA devices
 */
#define IRQ_PVC_TIMER0		(32 + 4)
#define IRQ_PVC_TIMER1		(32 + 5)
#define IRQ_PVC_TIMER2		(32 + 6)
#define IRQ_PVC_TIMER3		(32 + 7)
#define IRQ_PVC_TIMER4		(32 + 8)
#define IRQ_PVC_TIMER5		(32 + 9)
#define IRQ_PVC_TIMER6		(32 + 10)
#define IRQ_PVC_TIMER7		(32 + 11)
#define IRQ_PVC_TIMER8		(32 + 12)
#define IRQ_PVC_TIMER9		(32 + 13)
#define IRQ_PVC_TIMER10		(32 + 14)
#define IRQ_PVC_TIMER11		(32 + 15)
#define IRQ_PVC_TIMER12		(32 + 16)
#define IRQ_PVC_TIMER13		(32 + 17)
#define IRQ_PVC_TIMER14		(32 + 18)
#define IRQ_PVC_TIMER15		(32 + 19)
#define IRQ_PVC_TIMER16		(32 + 20)
#define IRQ_PVC_TIMER17		(32 + 21)
#define IRQ_PVC_TIMER18		(32 + 22)
#define IRQ_PVC_TIMER19		(32 + 23)
#define IRQ_PVC_TIMER20		(32 + 24)
#define IRQ_PVC_TIMER21		(32 + 25)
#define IRQ_PVC_TIMER22		(32 + 26)
#define IRQ_PVC_TIMER23		(32 + 27)
#define IRQ_PVC_TIMER24		(32 + 28)
#define IRQ_PVC_TIMER25		(32 + 29)

#define IRQ_PVC_UART0		{ (32 + 53) }
#define IRQ_PVC_UART1		{ (32 + 54) }
#define IRQ_PVC_UART2		{ (32 + 264) }
#define IRQ_PVC_UART3		{ (32 + 265) }

/* SDIP Interrupt(Use SDIO) */
#if defined(CONFIG_ARCH_PVC02V)
#define IRQ_PVC_SDC0_SDIP		87
#define IRQ_PVC_SDC0_SDIP2		88
#define IRQ_PVC_SDC1_SDIP		89
#define IRQ_PVC_SDC1_SDIP2		90
#define IRQ_PVC_SDC2_SDIP		91
#define IRQ_PVC_SDC2_SDIP2		92
#define IRQ_PVC_SDC3_SDIP		93
#define IRQ_PVC_SDC3_SDIP2		94
#endif

#define NR_IRQS			320

/*
 * Clock source
 */
#define PVC_PERI_CLK		13500000 /* 13.5MHz */
#define PVC_PERI_TIMER_CLK	13500000 /* 13.5MHz */

#endif
