#ifndef __MACH_CT_CA15_A7_H
#define __MACH_CT_CA15_A7_H

/* Versatile peripherals static mapping */

#define V2M_PA_CS0	0x08000000
#define V2M_PA_CS1	0x14000000
#define V2M_PA_CS2	0x18000000
#define V2M_PA_CS3	0x1c000000
#define V2M_PA_CS4	0x0c000000
#define V2M_PA_CS5	0x10000000

#define V2M_GIC_BASE		0xfe000000		/* V.A. */
#define V2M_GIC_START		0x2c000000		/* P.A. */
#define V2M_GIC_SIZE		0x00008000

#define V2M_PERI_BASE		0xfe200000		/* V.A. */
#define V2M_PERI_START		V2M_PA_CS3		/* P.A. */
#define V2M_PERI_SIZE		SZ_2M
#define V2M_PERI_VADDR(paddr)	((paddr) - V2M_PERI_START + V2M_PERI_BASE)

#define PVC_KERNEL_UNCACHE_BASE		0xfe400000			/* V.A. */
#define PVC_KERNEL_UNCACHE_START	CONFIG_KERNEL_RAM_BASE_ADDRESS	/* P.A. */
#define PVC_KERNEL_UNCACHE_SIZE		0x00004000

/* GIC addresses */
#define V2M_GIC_CPU_PADDR	(V2M_GIC_START + 0x00002000)
#define V2M_GIC_CPU_VADDR	(V2M_GIC_BASE  + 0x00002000)
#define V2M_GIC_DIST_PADDR	(V2M_GIC_START + 0x00001000)
#define V2M_GIC_DIST_VADDR	(V2M_GIC_BASE  + 0x00001000)
#define V2M_GIC_CPU_OFFSET	0x00000000

/* UART addresses */
#define V2M_UART0_PADDR		(V2M_PA_CS3 + 0x00090000)
#define V2M_UART1_PADDR		(V2M_PA_CS3 + 0x000a0000)
#define V2M_UART2_PADDR		(V2M_PA_CS3 + 0x000b0000)
#define V2M_UART3_PADDR		(V2M_PA_CS3 + 0x000c0000)

/* Timer addresses */
#define V2M_TIMER_0_1_PADDR	(V2M_PA_CS3 + 0x00110000)
#define V2M_TIMER_2_3_PADDR	(V2M_PA_CS3 + 0x00120000)

/* UART definitions for early kernel messages */
#if defined(CONFIG_PVC_EARLY_UART0)
#define PVC02V_DEBUG_UART_PADDR	V2M_UART0_PADDR
#define PVC02V_DEBUG_UART_VADDR	V2M_PERI_VADDR(V2M_UART0_PADDR)
#elif defined(CONFIG_PVC_EARLY_UART1)
#define PVC02V_DEBUG_UART_PADDR	V2M_UART1_PADDR
#define PVC02V_DEBUG_UART_VADDR	V2M_PERI_VADDR(V2M_UART1_PADDR)
#elif defined(CONFIG_PVC_EARLY_UART2)
#define PVC02V_DEBUG_UART_PADDR	V2M_UART2_PADDR
#define PVC02V_DEBUG_UART_VADDR	V2M_PERI_VADDR(V2M_UART2_PADDR)
#elif defined(CONFIG_PVC_EARLY_UART3)
#define PVC02V_DEBUG_UART_PADDR	V2M_UART3_PADDR
#define PVC02V_DEBUG_UART_VADDR	V2M_PERI_VADDR(V2M_UART3_PADDR)
#else /* default */
#define PVC02V_DEBUG_UART_PADDR	V2M_UART0_PADDR
#define PVC02V_DEBUG_UART_VADDR	V2M_PERI_VADDR(V2M_UART0_PADDR)
#endif

/* SMP synchronous flag */
#define PVC_SMP_SYNC_PADDR	(PVC_KERNEL_UNCACHE_START + 0x3000)
#define PVC_SMP_SYNC_VADDR	(PVC_KERNEL_UNCACHE_BASE  + 0x3000)

/*
 * SYSREG addresses
 */
#define V2M_SYSREGS		(V2M_PA_CS3 + 0x00010000)
#define V2M_SYSCTL		(V2M_PA_CS3 + 0x00020000)

/*
 * Offsets from SYSREGS base
 */
#define V2M_SYS_ID		0x000
#define V2M_SYS_SW		0x004
#define V2M_SYS_LED		0x008
#define V2M_SYS_100HZ		0x024
#define V2M_SYS_FLAGS		0x030
#define V2M_SYS_FLAGSSET	0x030
#define V2M_SYS_FLAGSCLR	0x034
#define V2M_SYS_NVFLAGS		0x038
#define V2M_SYS_NVFLAGSSET	0x038
#define V2M_SYS_NVFLAGSCLR	0x03c
#define V2M_SYS_MCI		0x048
#define V2M_SYS_FLASH		0x03c
#define V2M_SYS_CFGSW		0x058
#define V2M_SYS_24MHZ		0x05c
#define V2M_SYS_MISC		0x060
#define V2M_SYS_DMA		0x064
#define V2M_SYS_PROCID0		0x084
#define V2M_SYS_PROCID1		0x088
#define V2M_SYS_CFGDATA		0x0a0
#define V2M_SYS_CFGCTRL		0x0a4
#define V2M_SYS_CFGSTAT		0x0a8

/*
 * Configuration
 */
#define SYS_CFG_START		(1 << 31)
#define SYS_CFG_WRITE		(1 << 30)
#define SYS_CFG_OSC		(1 << 20)
#define SYS_CFG_VOLT		(2 << 20)
#define SYS_CFG_AMP		(3 << 20)
#define SYS_CFG_TEMP		(4 << 20)
#define SYS_CFG_RESET		(5 << 20)
#define SYS_CFG_SCC		(6 << 20)
#define SYS_CFG_MUXFPGA		(7 << 20)
#define SYS_CFG_SHUTDOWN	(8 << 20)
#define SYS_CFG_REBOOT		(9 << 20)
#define SYS_CFG_DVIMODE		(11 << 20)
#define SYS_CFG_POWER		(12 << 20)
#define SYS_CFG_SITE(n)		((n) << 16)
#define SYS_CFG_SITE_MB		0
#define SYS_CFG_SITE_DB1	1
#define SYS_CFG_SITE_DB2	2
#define SYS_CFG_STACK(n)	((n) << 12)

#define SYS_CFG_ERR		(1 << 1)
#define SYS_CFG_COMPLETE	(1 << 0)

#ifndef __ASSEMBLY__
int v2m_cfg_write(u32 devfn, u32 data);
int v2m_cfg_read(u32 devfn, u32 *data);
#endif /* !__ASSEMBLY__ */

/*
 * Interrupts.  Those in {} are for AMBA devices
 */
#define IRQ_V2M_WDT		{ (32 + 0) }
#define IRQ_V2M_TIMER0		(32 + 2)
#define IRQ_V2M_TIMER1		(32 + 2)
#define IRQ_V2M_TIMER2		(32 + 3)
#define IRQ_V2M_TIMER3		(32 + 3)
#define IRQ_V2M_RTC		{ (32 + 4) }
#define IRQ_V2M_UART0		{ (32 + 5) }
#define IRQ_V2M_UART1		{ (32 + 6) }
#define IRQ_V2M_UART2		{ (32 + 7) }
#define IRQ_V2M_UART3		{ (32 + 8) }
#define IRQ_V2M_MMCI		{ (32 + 9), (32 + 10) }
#define IRQ_V2M_AACI		{ (32 + 11) }
#define IRQ_V2M_KMI0		{ (32 + 12) }
#define IRQ_V2M_KMI1		{ (32 + 13) }
#define IRQ_V2M_CLCD		{ (32 + 14) }
#define IRQ_V2M_LAN9118		(32 + 15)
#define IRQ_V2M_ISP1761		(32 + 16)
#define IRQ_V2M_PCIE		(32 + 17)

#define NR_IRQS			192

/*
 * Clock source
 */
#define PVC_PERI_CLK		24000000 /* 24MHz */
#define PVC_PERI_TIMER_CLK	1000000 /* 1MHz */

#endif
