/* UniPhier On-board interrupt controller registers
 *
 * Copyright (C) 2007 Red Hat, Inc. All Rights Reserved.
 * Written by David Howells (dhowells@redhat.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public Licence
 * as published by the Free Software Foundation; either version
 * 2 of the Licence, or (at your option) any later version.
 */
#ifndef _ASM_ARCH_INTCTL_REGS_H
#define _ASM_ARCH_INTCTL_REGS_H

#ifdef __KERNEL__

#include <unit/intctl-regs.h>

#define PSR_PRI_BIT	0x00f00000
#define PSR_PRI_SHIFT	20

#define UNIPHIER_GIC_PRI_SHIFT	4

#ifndef __ASSEMBLY__
extern void mn_intc_set_level(unsigned int irq, unsigned int level);
extern void mn_intc_clear(unsigned int irq);
extern void mn_intc_set(unsigned int irq);
extern void mn_intc_enable(unsigned int irq);
extern void mn_intc_disable(unsigned int irq);
extern void mn_set_lateack_irq_type(int irq);
extern void mn_set_empty_irq_type(int irq);
extern unsigned int __mn_intc_get_mask_priority(void);
extern unsigned int mn_intc_get_mask_priority(void);
extern void __mn_intc_set_mask_priority(unsigned int pri);
extern void mn_intc_set_mask_priority(unsigned int pri);
extern unsigned int mn_intc_set_and_get_mask_priority(unsigned int pri);
#define ClearGicr(irq)			mn_intc_clear(irq)			/* deprecated */
#define SetLevelGicr(irq, level)	mn_intc_set_level((irq), (level))	/* deprecated */
#define EnableGicr(irq)			mn_intc_enable(irq)			/* deprecated */
#define DisableGicr(irq)		mn_intc_disable(irq)			/* deprecated */

#ifdef CONFIG_LINUX_CLI_MASK

#ifdef CONFIG_LINUX_CLI_MASK_DEBUG

#define FI_PRINT(x)				\
do {						\
	char *fi_msg = (x);			\
	while (*fi_msg) {			\
		outl(*fi_msg, FI_PRINT_UART);	\
		fi_msg++;			\
	}					\
	outl('\r', FI_PRINT_UART);		\
	outl('\n', FI_PRINT_UART);		\
} while (0)

#define FI_ASSERT(cond)				\
do {						\
	if (!(cond)) {				\
		FI_BUG("FI_ASSERT : " #cond);	\
	}					\
} while (0)

#define FI_CHECK_STATE(ibit, low, high)				\
do {								\
	unsigned long flags, pri;				\
	local_save_flags(flags);				\
	if ((ibit) >= 0) {					\
		FI_ASSERT(!!((flags) & PSR_I_BIT) == (ibit));	\
	}							\
	if ((low) >= 0) {					\
		pri = get_flags_intc_pri(flags);		\
		FI_ASSERT((pri <= (low)) && (pri >= (high)));	\
	}							\
} while (0)
#else /* CONFIG_LINUX_CLI_MASK_DEBUG */
#define FI_PRINT(x)			do { } while (0)
#define FI_ASSERT(cond)			do { } while (0)
#define FI_CHECK_STATE(ibit, low, high)	do { } while (0)
#endif /* CONFIG_LINUX_CLI_MASK_DEBUG */

#define FI_BUG(x)					\
do {							\
	FI_PRINT(x);					\
	__asm__ __volatile__ ("101:	b	101b");	\
} while (0)

extern int set_fastintr_handler(unsigned int,
			void (*)(unsigned int, struct pt_regs *));
extern void mn_fastintr_clear(unsigned int irq);
extern unsigned int get_current_irq_priority(void);

#include <linux/init.h>
extern void __init init_fastintr(void);
extern void __cpuinit init_fastintr_call_function_ipi(void);
extern int fastintr_smp_call_function_single(int cpu, void (*func) (void *info),
					     void *info, int wait);

#define local_irq_enable_intc_mask()						\
do {										\
	FI_CHECK_STATE(0, CONFIG_LINUX_CLI_LEVEL, CONFIG_LINUX_CLI_LEVEL);	\
	__mn_intc_set_mask_priority(get_current_irq_priority());		\
	FI_CHECK_STATE(0, CONFIG_LINUX_TASK_LEVEL - 1, CONFIG_LINUX_CLI_LEVEL);	\
} while (0)
#endif /* CONFIG_LINUX_CLI_MASK */

#endif /* __ASSEMBLY__ */

#endif /* __KERNEL__ */

#endif /* _ASM_ARCH_INTCTL_REGS_H */
