/* MN2PS On-chip serial registers
 *
 * Copyright (C) 2012 Panasonic Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public Licence
 * as published by the Free Software Foundation; either version
 * 2 of the Licence, or (at your option) any later version.
 */
#ifndef _UNIT_SERIAL_MN2PS_H
#define _UNIT_SERIAL_MN2PS_H

#ifdef __KERNEL__

#include <unit/serial_mn2ps_regs.h>

#define SERIAL_FIFO_SIZE	128

#define SERIAL_TX_THRESHOLD_VALUE	128
#define SERIAL_RX_THRESHOLD_VALUE	1

struct serial_mn2ps_port {
	const char		*name;		/* name of serial port */
	const char		*rx_name;	/* Rx interrupt handler name of serial port */
	const char		*tx_name;	/* Tx interrupt handler name of serial port */
	const char		*tm_name;	/* Timer interrupt handler name */
	unsigned short		type;		/* type of serial port */

	struct uart_port	uart;

	/* serial register */
	unsigned long           exclksel_mapbase;
	unsigned long           delay_mapbase;
	unsigned long __iomem *	exclksel_membase;
	unsigned long __iomem *	delay_membase;

	u8			tx_xchar;	/* high-priority XON/XOFF buffer */
	unsigned short		tx_break;	/* current break reception status */
	int			rx_irq;		/* reception IRQ */
	int			tx_irq;		/* transmission IRQ */

	u16			options;	/* options */

	unsigned long		ioclk;		/* base clock rate */
};


void serial_mn2ps_direct_console_write(struct serial_mn2ps_port *,
					const char *, unsigned);
void serial_mn2ps_fifo_console_write(struct serial_mn2ps_port *,
					const char *, unsigned);
int serial_mn2ps_ioremap(struct serial_mn2ps_port* port);

extern const struct uart_ops serial_mn2ps_direct_ops;
extern const struct uart_ops serial_mn2ps_fifo_ops;

#endif

#endif /* _UNIT_SERIAL_MN2PS_H */
