/* MN2PS On-chip serial registers
 *
 * Copyright (C) 2012 Panasonic Corporation.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public Licence
 * as published by the Free Software Foundation; either version
 * 2 of the Licence, or (at your option) any later version.
 */
#ifndef _MN2PS_SERIAL_REGS_H
#define _MN2PS_SERIAL_REGS_H

#ifdef __KERNEL__

#define NR_PORTS                4

/* register base address */
#define SC0_BASE                0x9F000800
#define SC1_BASE                0x9F000814
#define SC2_BASE                0x9F000c00
#define SC3_BASE                0x9F000c40
#define SC0_EXCLK_SEL_BASE      0x9F000840       /* clock selecter0     */
#define SC1_EXCLK_SEL_BASE      0x9F000844       /* clock selecter1     */
#define SC_TX_DELAY1_BASE       0x9F000858       /* trabsnut delay      */
#define SC_MAP_BASE             SC0_BASE

#define SC0_BASE_OFFSET         0
#define SC1_BASE_OFFSET         (SC1_BASE - SC0_BASE)
#define SC2_BASE_OFFSET         (SC2_BASE - SC0_BASE)
#define SC3_BASE_OFFSET         (SC3_BASE - SC0_BASE)
#define SC0_EXCLK_SEL_BASE_OFFSET      (SC0_EXCLK_SEL_BASE - SC0_BASE)
#define SC1_EXCLK_SEL_BASE_OFFSET      (SC1_EXCLK_SEL_BASE - SC0_BASE)
#define SC_TX_DELAY1_BASE_OFFSET       (SC_TX_DELAY1_BASE - SC0_BASE)

/*
 * Serial port 0-1
 */

/* register offset address */
#define SC0CTR                  0x00000000       /* control reg         */
#define SC0RXB                  0x00000004       /* receive buffer reg  */
#define SC0TXB                  0x00000008       /* transmit buffer reg */
#define SC0STR                  0x0000000C       /* status reg          */
#define SIOTOP_CYCLE0           0x00000010       /* prescaler cycle     */

/* register area size */
#define SC_MAP_SIZE             0x1000
#define SC_EXCLK_SEL_SIZE       4
#define SC_TX_DELAY_SIZE        4

/* intr_flags bits */
#define MNSCx_RX_AVAIL          0x01
#define MNSCx_RX_OVERF          0x02
#define MNSCx_TX_SPACE          0x04
#define MNSCx_TX_EMPTY          0x08

#define SC01CTR_CK              0x0007  /* clock source select          */
#define SC01CTR_CK_IOCLK_2      0x0001  /* - 1/2 IOCLK                  */
#define SC01CTR_CK_IOCLK_16     0x0003  /* - 1/16 IOCLK                 */
#define SC01CTR_STB             0x0008  /* stop bit select              */
#define SC01CTR_STB_1BIT        0x0000  /* - 1 stop bit                 */
#define SC01CTR_STB_2BIT        0x0008  /* - 2 stop bits                */
#define SC01CTR_PB              0x0070  /* parity bit select            */
#define SC01CTR_PB_NONE         0x0000  /* - no parity                  */
#define SC01CTR_PB_FIXED0       0x0040  /* - fixed at 0                 */
#define SC01CTR_PB_FIXED1       0x0050  /* - fixed at 1                 */
#define SC01CTR_PB_EVEN         0x0060  /* - even parity                */
#define SC01CTR_PB_ODD          0x0070  /* - odd parity                 */
#define SC01CTR_CLN             0x0080  /* character length             */
#define SC01CTR_CLN_7BIT        0x0000  /* - 7 bit chars                */
#define SC01CTR_CLN_8BIT        0x0080  /* - 8 bit chars                */
#define SC01CTR_IIC             0x0100  /* I2C mode select              */
#define SC01CTR_OD              0x0200  /* bit order select             */
#define SC01CTR_OD_LSBFIRST     0x0000  /* - LSB first                  */
#define SC01CTR_OD_MSBFIRST     0x0200  /* - MSB first                  */
#define SC01CTR_TI              0x0400  /* transmit irq request         */
#define SC01CTR_TI_DONE         0x0000  /* transmit done request        */
#define SC01CTR_TI_EMPTY        0x0400  /* transfer buffer empty request*/
#define SC01CTR_MD              0x0800  /* mode select                  */
#define SC01CTR_MD_STST_SYNC    0x0000  /* - start-stop synchronous     */
#define SC01CTR_MD_CLOCK_SYNC   0x0800  /* - clock synchronous          */
#define SC01CTR_BKE             0x2000  /* break transmit enable        */
#define SC01CTR_RXE             0x4000  /* receive enable               */
#define SC01CTR_TXE             0x8000  /* transmit enable              */

#define SC01STR_OEF             0x0100  /* overrun error found          */
#define SC01STR_PEF             0x0200  /* parity error found           */
#define SC01STR_FEF             0x0400  /* framing error found          */
#define SC01STR_RXA             0x1000  /* receive buffer status        */
#define SC01STR_RXA_NONE        0x0000  /*  - not receive data          */
#define SC01STR_RXA_AVAIL       0x1000  /*  - available receive data    */
#define SC01STR_TBF             0x2000  /* transmit buffer status       */
#define SC01STR_TBF_FULL        0x2000  /*  - transmit buffer full      */
#define SC01STR_TBF_EMPTY       0x0000  /*  - transmit buffer empty     */
#define SC01STR_RBSY            0x4000  /* receive status               */
#define SC01STR_RBSY_IDLE       0x0000  /*  - receive status idle       */
#define SC01STR_RBSY_BUSY       0x4000  /*  - receive status busy       */
#define SC01STR_TBSY            0x8000  /* transmit status              */
#define SC01STR_TBSY_IDLE       0x0000  /*  - transmit status idle      */
#define SC01STR_TBSY_BUSY       0x8000  /*  - transmit status busy      */
#define SC01STR_ERROR_BITS      0x0700  /* FE,PE,OE                     */

#define SC0_EXCLK_SEL_TM8       0x0000  /* - TM8 clock                  */
#define SC0_EXCLK_SEL_TIMER     0x0002  /* - TIMER(prescaler) clock     */
#define SC0_EXCLK_SEL_SYSCLK    0x0003  /* - sysclk                     */

/*
 * Serial port 2-3 (FIFO)
 */

/* register area size */
#define SC_FIFO_SIZE            (SC3_BASE - SC2_BASE)

/* register offset address */
#define SIOTOP_CNFIGN           0x00000000        /* config reg */
#define SIOTOP_CYCLEN           0x00000004        /* EXCLK reg */
#define SIOTOP_RXDATAN          0x00000008        /* receive buffer reg */
#define SCNRXB                  SIOTOP_RXDATAN    /* receive buffer reg(alias) */
#define SIOTOP_RXTVAN           0x0000000C        /* receive buffer remain reg */
#define SIOTOP_RXTRSN           0x00000010        /* receive buffer threshold reg */
#define SIOTOP_RXCNTN           0x00000014        /* receive control reg */
#define SIOTOP_TXDATAN          0x00000018        /* transmit buffer reg */
#define SCNTXB                  SIOTOP_TXDATAN    /* transmit buffer reg(alias) */
#define SIOTOP_TXTVAN           0x0000001C        /* transmit buffer reg */
#define SIOTOP_TXTRSN           0x00000020        /* transmit buffer threshold reg */
#define SIOTOP_TXCNTN           0x00000024        /* transmit control reg */
#define SIOTOP_SCACTRN          0x00000028        /* control reg */
#define SCNCTR                  SIOTOP_SCACTRN    /* control reg(alias) */
#define SIOTOP_SCASTRN          0x0000002C        /* store reg */
#define SIOTOP_SCARBN           0x00000030        /* receive buffer reg(I2C) */
#define SIOTOP_SCATBN           0x00000034        /* transmit buffer reg(I2C) */
#define SIOTOP_STATUSN          0x00000038        /* status reg */

/* SIOTOP_CNFIGN reg bit */
#define SIOTOP_CNFIGN_BYPASS    0x8000            /* bypas switch bit */
#define SIOTOP_CNFIGN_MODEM     0x3000            /* trans mode bit */
#define SIOTOP_CNFIGN_MODEM_ASC 0x0000            /* - start-stop synchronous */
#define SIOTOP_CNFIGN_MODEM_SSC 0x1000            /* - clock synchronous  */
#define SIOTOP_CNFIGN_RESET     0x0100            /* IP reset bit */
#define SIOTOP_CNFIGN_CLK       0x0010            /* SSC receive signal*/
#define SIOTOP_CNFIGN_CLK_DOWN  0x0000            /* - down edge signal */
#define SIOTOP_CNFIGN_CLK_UP    0x0010            /* - up edge signal */
#define SIOTOP_CNFIGN_RXC       0x0008            /* rxc input select */
#define SIOTOP_CNFIGN_RXC_GIO   0x0000            /* - GIO_CK_I */
#define SIOTOP_CNFIGN_RXC_TXC   0x0008            /* - txc */
#define SIOTOP_CNFIGN_COE       0x0004            /* SIO_CK_O output enable */
#define SIOTOP_CNFIGN_COE_IN    0x0000            /* - input */
#define SIOTOP_CNFIGN_COE_PUT   0x0004            /* - output */
#define SIOTOP_CNFIGN_DIR       0x0002            /* data input setting */
#define SIOTOP_CNFIGN_DIR_DO    0x0000            /* - GIO_DO_I */
#define SIOTOP_CNFIGN_DIR_DI    0x0002            /* - GIO_DI_I */
#define SIOTOP_CNFIGN_DOE       0x0001            /* data output enable */

/* SIOTOP_RXCNTN reg bit */
#define SIOTOP_RXCNTN_EI        0x0080            /* interrupt enable bit */
#define SIOTOP_RXCNTN_FIFO_CLR  0x0002            /* FIFO clear bit */
#define SIOTOP_RXCNTN_RXD_EN    0x0001            /* receive enable */

/* SIOTOP_TXCNTN reg bit */
#define SIOTOP_TXCNTN_EI_FIN    0x8000            /* interrupt enable bit for bug fix */
#define SIOTOP_TXCNTN_EI        0x0080            /* interrupt enable bit */
#define SIOTOP_TXCNTN_FIFO_CLR  0x0002            /* FIFO clear bit */
#define SIOTOP_TXCNTN_TXD_EN    0x0001            /* transmit enable */

/* SIOTOP_SCACTRN reg bit */
#define SIOTOP_SCACTRN_SCATEN   0x8000            /* transmit enable */
#define SIOTOP_SCACTRN_SCAREN   0x4000            /* receive enable */
#define SIOTOP_SCACTRN_SCABRE   0x2000            /* break transmit enable */
#define SIOTOP_SCACTRN_SCAPTL   0x0800            /* mode select */
#define SIOTOP_SCACTRN_SCAPTL_SYNC              0x0000          /* - start-stop synchronous */
#define SIOTOP_SCACTRN_SCAPTL_CLOCK_SYNC        0x0800          /* - clock synchronous  */
#define SIOTOP_SCACTRN_SCAOB            0x0200                  /* bit order select */
#define SIOTOP_SCACTRN_SCAOB_LSB        0x0000                  /* - LSB first */
#define SIOTOP_SCACTRN_SCAOB_MSB        0x0200                  /* - MSB first */
#define SIOTOP_SCACTRN_CLN              0x0080                  /* character length */
#define SIOTOP_SCACTRN_CLN_7BIT         0x0000                  /* - 7 bit chars */
#define SIOTOP_SCACTRN_CLN_8BIT         0x0080                  /* - 8 bit chars */
#define SIOTOP_SCACTRN_SCAPTY           0x0070                  /* parity bit select */
#define SIOTOP_SCACTRN_NONE             0x0000                  /* - no parity */
#define SIOTOP_SCACTRN_FIXED0           0x0040                  /* - fixed at 0 */
#define SIOTOP_SCACTRN_FIXED1           0x0050                  /* - fixed at 1 */
#define SIOTOP_SCACTRN_EVEN             0x0060                  /* - even parity */
#define SIOTOP_SCACTRN_ODD              0x0070                  /* - odd parity */
#define SIOTOP_SCACTRN_STB              0x0008                  /* stop bit select */
#define SIOTOP_SCACTRN_STB_1BIT         0x0000                  /* - 1 stop bit */
#define SIOTOP_SCACTRN_STB_2BIT         0x0008                  /* - 2 stop bits */
#define SIOTOP_SCACTRN_CK               0x0003                  /* clock source select */
#define SIOTOP_SCACTRN_CK_IOCLK_2       0x0001                  /* - 1/2 IOCLK */
#define SIOTOP_SCACTRN_CK_IOCLK_16      0x0003                  /* - 1/16 IOCLK */

/* SIOTOP_SCASTRN reg bit */
#define SIOTOP_SCASTRN_TBSY             0x8000                  /* transmit state */
#define SIOTOP_SCASTRN_TBSY_OK          0x0000                  /* - transmit enable */
#define SIOTOP_SCASTRN_TBSY_BSY         0x8000                  /* - busy */
#define SIOTOP_SCASTRN_RBSY             0x4000                  /* receive state */
#define SIOTOP_SCASTRN_RBSY_OK          0x0000                  /* - receive enable */
#define SIOTOP_SCASTRN_RBSY_BSY         0x4000                  /* - busy */
#define SIOTOP_SCASTRN_RXA              0x1000                  /* receive buffer status */
#define SIOTOP_SCASTRN_RXA_NONE         0x0000                  /* - not receive data */
#define SIOTOP_SCASTRN_RXA_AVAIL        0x1000                  /* - available receive data */
#define SIOTOP_SCASTRN_SCAFE            0x0400                  /* framing error flg */
#define SIOTOP_SCASTRN_SCAPE            0x0200                  /* parity error flg */
#define SIOTOP_SCASTRN_SCAOE            0x0100                  /* over run error flg */
#define SIOTOP_SCASTRN_ERROR_BITS       0x0700                  /* FE,PE,OE */

/* SIOTOP_STATUSN reg bit */
#define SIOTOP_STATUSN_TXDIR    0x0200                          /* transmit interrupt */
#define SIOTOP_STATUSN_RXDIR    0x0100                          /* receive interrupt */
#define SIOTOP_STATUSN_RXDFOR   0x0040                          /* overrun error */
#define SIOTOP_STATUSN_SCAFECLR 0x0008                          /* framing error */
#define SIOTOP_STATUSN_SCAPECLR 0x0004                          /* parity error */
#define SIOTOP_STATUSN_ERROR_BITS       0x340                   /* error bit */

#define SC0RXIRQ                113     /* timer 0 Receive IRQ          */
#define SC0TXIRQ                114     /* timer 0 Transmit IRQ         */
#define SC1RXIRQ                115     /* timer 1 Receive IRQ          */
#define SC1TXIRQ                116     /* timer 1 Transmit IRQ         */
#define SC2RXIRQ                117     /* serial 2 Receive IRQ */
#define SC2TXIRQ                118     /* serial 2 Transmit IRQ */
#define SC3RXIRQ                119     /* serial 3 Receive IRQ */
#define SC3TXIRQ                120     /* serial 3 Transmit IRQ */

#define SERIAL_PORT_INDEX(n)    (n - 2)

#endif

#endif /* _MN2PS_SERIAL_REGS_H */
