/*
 *  arch/arm/mach-uniphier/unit-mn2ps015_ref/include/unit/timex.h
 *
 *  Copyright (C) 2011 Panasonic Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef __ASM_UNIT_TIMEX_H
#define __ASM_UNIT_TIMEX_H

#ifndef __ASM_ARCH_TIMEX_H
#error only <mach/timex.h> can be included directly
#endif

#ifndef __ASSEMBLY__
#include <asm/io.h>
#endif /* __ASSEMBLY__ */

#include <mach/timer-regs.h>
#include <mach/clock.h>
#include <asm/param.h>

/*
 * jiffies counter specifications
 */
#define	TMJCBR_MAX		0x000ffff0	/* 20bit */
/* If TMJCBR_MAX = 0x000fffff, a counter may be overflowed in reload_jiffies_counter(). */

#define	TMJCIRQ			TM16_13AIRQ

#ifndef __ASSEMBLY__

#define MN_SRC_IOBCLK	(MN_PERI_CLK / 2)

#ifndef HZ
# error HZ undeclared.
#endif /* !HZ */

#define MN_JCCLK		(MN_SRC_IOBCLK)
#define MN_TSCCLK		(MN_SRC_IOBCLK)

#define MN_JC_PER_HZ		((MN_JCCLK + HZ / 2) / HZ)
#define MN_TSC_PER_HZ		((MN_TSCCLK + HZ / 2) / HZ)

/* Check bit width of MTM interval value that sets base register */
#if (MN_JC_PER_HZ - 1) > TMJCBR_MAX
# error tick timer interval value is overflow.
#endif

/*
 * Set 8bit timer register functions.
 *
 * Each register contain some timer's bit.
 * It need to spinlock when the register accessed by multi CPUs,
 * However, MN2PS015 is not SMP.
 */
static inline void tm8_set_md(u32 set, int timer)
{
	unsigned long flags;
	u32 tmp32;
	u32 tmmd = TM8_X_MD(timer);

	local_irq_save(flags);

	tmp32 = inl(tmmd);
	TM8_BIT_CLEAR(tmp32, timer);
	TM8_BIT_SET(tmp32, set, timer);
	outl(tmp32, tmmd);
	tmp32 = inl(tmmd);

	local_irq_restore(flags);
}

static inline void tm8_set_br(u32 set, int timer)
{
	unsigned long flags;
	u32 tmp32;
	u32 tmbr = TM8_X_BR(timer);

	local_irq_save(flags);

	tmp32 = inl(tmbr);
	TM8_BIT_CLEAR(tmp32, timer);
	TM8_BIT_SET(tmp32, set, timer);
	outl(tmp32, tmbr);
	tmp32 = inl(tmbr);

	local_irq_restore(flags);
}

/*
 * Set 16bit timer register functions.
 */
static inline void tm16_set_md(u32 set, int timer)
{
	u32 tmp32;

	outl(set, TM16_X_MD(timer));
	tmp32 = inl(TM16_X_MD(timer));
}

static inline void tm16_set_mdab(u32 set, int timer)
{
	u32 tmp32;

	outl(set, TM16_X_MDA_B(timer));
	tmp32 = inl(TM16_X_MDA_B(timer));
}

static inline void tm16_set_ca(u32 set, int timer)
{
	u32 tmp32;

	outl(set, TM16_X_CA(timer));
	tmp32 = inl(TM16_X_CA(timer));
}

static inline void tm16_set_cb(u32 set, int timer)
{
	u32 tmp32;

	outl(set, TM16_X_CB(timer));
	tmp32 = inl(TM16_X_CB(timer));
}

/*
 * Functions for clockevents timer.
 */

static inline void stop_jiffies_counter(void)
{
	tm16_set_md(0, 12);
	tm16_set_md(0, 13);
}

static inline void reload_jiffies_counter(u32 cnt)
{
	u32 t0, t1;
	u16 cnt0, cnt1;

	/* A lower counter (4bit) counts 0xf to 0 */
	cnt0 = 0xf;
	/* An upper counter (16bit) counts 0 to cnt1 */
	cnt1 = (cnt + 0xf) >> 4;

	/* Setup timer count value  for (cnt0 * cnt1) counts. */
	/*  (cnt0 * cnt1) = ((cnt + 0xf) & 0x000ffff0 */
	tm16_set_ca(cnt0, 12);
	tm16_set_ca(cnt1, 13);

	t0 = TM16_X_MD_CK_SYSCLK | TM16_X_MD_UD_DOWN | TM16_X_MD_CLE | TM16_X_MD_LDE;
	tm16_set_md(t0, 12);

	t1 = TM16_X_MD_CK_16UFLOW | TM16_X_MD_UD_UP | TM16_X_MD_LDE;
	tm16_set_md(t1, 13);

	t1 &= ~TM16_X_MD_LDE;
	t1 |= TM16_X_MD_CNE;
	tm16_set_md(t1, 13);

	t0 &= ~TM16_X_MD_LDE;
	t0 |= TM16_X_MD_CNE;
	tm16_set_md(t0, 12);
}

#endif /* !__ASSEMBLY__ */

#endif /* __ASM_UNIT_TIMEX_H */
