/*
 * linux/arch/arm/mach-uniphier/unit-mn2ps015_ref/time.c
 *
 * Copyright (C) 2012 Panasonic Corporation
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 */
#include <linux/init.h>
#include <linux/io.h>
#include <linux/clockchips.h>
#include <linux/cnt32_to_63.h>

#include <mach/hardware.h>

#define LOWER_TSCOUNTER	TM16_16_BC
#define UPPER_TSCOUNTER	TM16_17_BC

cycle_t uniphier_get_cycles(struct clocksource *cs)
{
	u16 upper0, upper1, lower;
	do {
		upper0 = (u16)inl(UPPER_TSCOUNTER);
		lower  = (u16)inl(LOWER_TSCOUNTER);
		upper1 = (u16)inl(UPPER_TSCOUNTER);
	} while (upper0 != upper1);

	return (cycle_t)~(((u32)upper0 << 16) | lower);
}

static struct clocksource clocksource_timer = {
	.name	= "TSC",
	.rating	= 200,
	.read	= uniphier_get_cycles,
	.mask	= CLOCKSOURCE_MASK(32),
	.flags	= CLOCK_SOURCE_IS_CONTINUOUS,
};

extern void clocksource_set_clock(struct clocksource *cs, unsigned int clock);

void __init init_clocksource(void)
{
	u32 t0, t1;

	/*
	 * Setup 16bit timer 16 & 17 cascaded as a 32-bit counter
	 * for free-running clocksource.
	 */

	t0 = TM16_X_MD_CK_SYSCLK | TM16_X_MD_UD_DOWN;
	t0 |= TM16_X_MD_LDE;
	tm16_set_md(t0, 16);

	t1 = TM16_X_MD_CK_16UFLOW | TM16_X_MD_UD_DOWN;
	t1 |= TM16_X_MD_LDE;
	tm16_set_md(t1, 17);

	t1 &= ~TM16_X_MD_LDE;
	t1 |= TM16_X_MD_CNE;
	tm16_set_md(t1, 17);

	t0 &= ~TM16_X_MD_LDE;
	t0 |= TM16_X_MD_CNE;
	tm16_set_md(t0, 16);

	clocksource_set_clock(&clocksource_timer, MN_TSCCLK);
	clocksource_register(&clocksource_timer);
}

/*
 * scheduler clock - returns current time in nanosec units.
 * This has a resolution of about 18.5ns,
 * and a maximum value of about 292 years.
 */
unsigned long long sched_clock(void)
{
	unsigned long long v;

	v = cnt32_to_63((u32)uniphier_get_cycles(&clocksource_timer));

	return v * NSEC_PER_SEC / MN_TSCCLK;
}
