/*
 *  arch/arm/mach-uniphier/unit-mn2ps015_ref/unit-init.c
 *
 *  Copyright (C) 2011 Panasonic Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/sysdev.h>
#include <linux/io.h>
#include <linux/irq.h>
#include <linux/clockchips.h>
#include <linux/interrupt.h>

#include <asm/setup.h>
#include <asm/irq.h>
#include <asm/leds.h>
#include <asm/mach-types.h>
#include <asm/pmu.h>
#include <asm/smp_twd.h>
#include <asm/localtimer.h>
#include <asm/hardware/gic.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/time.h>

#include <mach/hardware.h>
#include <mach/irqs.h>
#include <mach/serial.h>
#include <mach/timer-regs.h>

#include <asm/io.h>
#include <linux/syscalls.h>
#include <mach/intctl-regs.h>
#if defined(CONFIG_PVC01V_UNIT_SPRING13)
#include <mach/pvc01v.h>
#endif

struct machine_desc;

extern void __iomem *gic_cpu_base_addr;

#define head_cmdline ((char *)__phys_to_virt(PHYS_OFFSET+0x00003000))

static struct map_desc uniphier_io_desc[] __initdata = {
	{
		.virtual	= UNIPHIER_PRIVATE_IO_BASE,
		.pfn		= __phys_to_pfn(UNIPHIER_PRIVATE_IO_START),
		.length		= UNIPHIER_PRIVATE_IO_SIZE,
		.type		= MT_DEVICE,
	},
	{
		.virtual	= UNIPHIER_SSC_BASE0,
		.pfn		= __phys_to_pfn(UNIPHIER_SSC_START0),
		.length		= UNIPHIER_SSC_SIZE0,
		.type		= MT_DEVICE,
	},
	{
		.virtual	= UNIPHIER_SSC_BASE1,
		.pfn		= __phys_to_pfn(UNIPHIER_SSC_START1),
		.length		= UNIPHIER_SSC_SIZE1,
		.type		= MT_DEVICE,
	},
	{
		.virtual	= UNIPHIER_SSC_BASE2,
		.pfn		= __phys_to_pfn(UNIPHIER_SSC_START2),
		.length		= UNIPHIER_SSC_SIZE2,
		.type		= MT_DEVICE,
	},
	{
		.virtual	= UNIPHIER_VSSC_BASE0,
		.pfn		= __phys_to_pfn(UNIPHIER_VSSC_START0),
		.length		= UNIPHIER_VSSC_SIZE0,
		.type		= MT_DEVICE,
	},
#if 0
	{
		.virtual	= UNIPHIER_VSSC_BASE1,
		.pfn		= __phys_to_pfn(UNIPHIER_VSSC_START1),
		.length		= UNIPHIER_VSSC_SIZE1,
		.type		= MT_DEVICE,
	},
#endif
	{
		.virtual	= UNIPHIER_CA9P_BASE,
		.pfn		= __phys_to_pfn(UNIPHIER_CA9P_START),
		.length		= UNIPHIER_CA9P_SIZE,
		.type		= MT_DEVICE,
	},
	{
		.virtual	= UNIPHIER_KERNEL_UNCACHE_BASE,
		.pfn		= __phys_to_pfn(UNIPHIER_KERNEL_UNCACHE_START),
		.length		= UNIPHIER_KERNEL_UNCACHE_SIZE,
		.type		= MT_UNCACHED,
	},
	{
		.virtual	= UNIPHIER_KERNEL_UCWG_BASE,
		.pfn		= 0,
		.length		= UNIPHIER_KERNEL_UCWG_SIZE,
		.type		= MT_MEMORY_BUFFERABLE,
	},
};

extern unsigned long ucwg_dummy[];

static void __init uniphier_map_io(void)
{
	uniphier_io_desc[ARRAY_SIZE(uniphier_io_desc)-1].pfn
		= __phys_to_pfn(__pa((unsigned long)&ucwg_dummy)
				 & ~(UNIPHIER_KERNEL_UCWG_SIZE-1));
	iotable_init(uniphier_io_desc, ARRAY_SIZE(uniphier_io_desc));
}

static unsigned int irq_trigger_type[] __initdata =
{
IRQ_TYPE_EDGE_RISING,	/* 32 */
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_LEVEL_HIGH,	/* 40 */
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_LEVEL_HIGH,
IRQ_TYPE_LEVEL_HIGH,
IRQ_TYPE_LEVEL_HIGH,
IRQ_TYPE_LEVEL_HIGH,
IRQ_TYPE_LEVEL_LOW,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,	/* 50 */
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,	/* 60 */
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,	/* 70 */
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,	/* 80 */
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,	/* 90 */
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,	/* 100 */
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,	/* 110 */
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,	/* 120 */
IRQ_TYPE_LEVEL_HIGH,
IRQ_TYPE_LEVEL_HIGH,
IRQ_TYPE_LEVEL_HIGH,
IRQ_TYPE_LEVEL_HIGH,
IRQ_TYPE_LEVEL_LOW,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,	/* 130 */
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,	/* 140 */
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,	/* 150 */
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,	/* 160 */
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,	/* 170 */
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_LEVEL_HIGH,
IRQ_TYPE_LEVEL_HIGH,
IRQ_TYPE_LEVEL_HIGH,
IRQ_TYPE_LEVEL_HIGH,
IRQ_TYPE_LEVEL_HIGH,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,	/* 180 */
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_LEVEL_LOW,
IRQ_TYPE_EDGE_RISING,	/* 190 */
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,	/* 200 */
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,	/* 210 */
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,	/* 220 */
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,	/* 230 */
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,	/* 240 */
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_EDGE_RISING,
IRQ_TYPE_LEVEL_HIGH,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,	/* 250 */
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
IRQ_TYPE_EDGE_FALLING,
};

static struct resource pmu_resources[] = {
	[0] = {
		.start		= IRQ_UNIPHIER_PMU_CPU,
		.end		= IRQ_UNIPHIER_PMU_CPU,
		.flags		= IORESOURCE_IRQ,
	},
};

static struct platform_device pmu_device = {
	.name			= "arm-pmu",
	.id			= ARM_PMU_DEVICE_CPU,
	.num_resources		= ARRAY_SIZE(pmu_resources),
	.resource		= pmu_resources,
};

static void __init gic_init_irq(void)
{
	int i;
#ifdef CONFIG_LINUX_CLI_MASK
	init_fastintr();
#endif /* CONFIG_LINUX_CLI_MASK */
	gic_cpu_base_addr = (void __iomem *)UNIPHIER_GIC_CPU_BASE;
	gic_dist_init(0, (void __iomem *)UNIPHIER_GIC_DIST_BASE, 29);
	for (i = 0;i < sizeof(irq_trigger_type)/sizeof(unsigned int);i++) {
		unsigned int type = irq_trigger_type[i];
		if (type == IRQ_TYPE_EDGE_RISING || type == IRQ_TYPE_EDGE_FALLING) {
			set_irq_type(IRQ_GIC_START + i, type);
		}
	}
	gic_cpu_init(0, (void __iomem *)UNIPHIER_GIC_CPU_BASE);
}

extern void __init init_clocksource(void);
extern void __init init_clockevents(void);

static void __init uniphier_timer_init(void)
{
	init_clocksource();

	init_clockevents();
}

static struct sys_timer uniphier_timer = {
	.init		= uniphier_timer_init,
};

extern unsigned int __atags_pointer;

static void __init uniphier_fixup(struct machine_desc *mdesc, struct tag *tags,
			       char **cmdline, struct meminfo *meminfo)
{
#if defined(CONFIG_PVC01V_UNIT_SPRING13)
	/* not replace command line. */
#else
#if !defined(CONFIG_XIP_KERNEL)
	if (__atags_pointer) {
		if (tags->hdr.tag == ATAG_CORE)
			return;
	}
	if ( head_cmdline[ 0] != '\0') {
		strncpy( *cmdline, head_cmdline, COMMAND_LINE_SIZE -1);
		tags->hdr.tag = ATAG_NONE;
	}
#endif
#endif
}

#if defined(CONFIG_PVC01V_UNIT_SPRING13)
static struct resource pvcsdio_resources[] = {
	[0] = {
		.start	= PVCSD_SDX_BASE,
		.end	= PVCSD_SDX_BASE + SZ_4K - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= IRQ_UNIPHIER_SDC0_SDIP,
		.end	= IRQ_UNIPHIER_SDC0_SDIP,
		.flags	= IORESOURCE_IRQ,
	},
	[2] = {
		.start	= IRQ_UNIPHIER_SDC0_SDIP2,
		.end	= IRQ_UNIPHIER_SDC0_SDIP2,
		.flags	= IORESOURCE_IRQ,
	},
	[3] = {
		.start	= IRQ_UNIPHIER_SDC1_SDIP,
		.end	= IRQ_UNIPHIER_SDC1_SDIP,
		.flags	= IORESOURCE_IRQ,
	},
	[4] = {
		.start	= IRQ_UNIPHIER_SDC1_SDIP2,
		.end	= IRQ_UNIPHIER_SDC1_SDIP2,
		.flags	= IORESOURCE_IRQ,
	},
	[5] = {
		.start	= IRQ_UNIPHIER_SDTOP0_DMA,
		.end	= IRQ_UNIPHIER_SDTOP0_DMA,
		.flags	= IORESOURCE_IRQ,
	},
	[6] = {
		.start	= IRQ_UNIPHIER_SDTOP1_DMA,
		.end	= IRQ_UNIPHIER_SDTOP1_DMA,
		.flags	= IORESOURCE_IRQ,
	},
};

static u64 pvcsdio_dmamask = 0xffffffffUL;

static struct platform_device pvc01v_sdio = {
	.name		= "pvsdio",
	.id		= -1,
	.dev		= {
		.dma_mask = &pvcsdio_dmamask,
		.coherent_dma_mask = 0xffffffff,
	},
	.num_resources	= ARRAY_SIZE(pvcsdio_resources),
	.resource	= pvcsdio_resources,
};

static struct pvcsdio_platform_data pvcsdio_data;
#endif /* CONFIG_PVC01V_UNIT_SPRING13 */

static void __init uniphier_init(void)
{
#ifdef CONFIG_LEDS
	leds_event = uniphier_leds_event;
#endif
	mn_intc_set_level(IRQ_UNIPHIER_PMU_CPU, 2);
	platform_device_register(&pmu_device);
#if defined(CONFIG_PVC01V_UNIT_SPRING13)
#if defined(CONFIG_MMC_PVC01V) || defined(CONFIG_MMC_PVC01V_MODULE)
	pvcsdio_data.clock = 43875000;
	pvc01v_sdio.dev.platform_data = &pvcsdio_data;
	platform_device_register(&pvc01v_sdio);
#endif
#endif
}

MACHINE_START(MN2PS015_REF, "Panasonic MN2PS015")
	/* Maintainer: Panasonic */
	.phys_io	= UNIPHIER_CA9P_START,
	.io_pg_offst	= (UNIPHIER_CA9P_BASE >> 18) & 0xfffc,
	.fixup		= uniphier_fixup,
	.map_io		= uniphier_map_io,
	.init_irq	= gic_init_irq,
	.timer		= &uniphier_timer,
	.init_machine	= uniphier_init,
MACHINE_END

static int __init unit_late_init(void)
{
	return 0;
}

late_initcall( unit_late_init );
