/**
 *  Copyright (C) 2012 Panasonic Corporation. All Rights Reserved.
 *
 * @file
 * @brief   CPU間通信D/D定義ヘッダー
 */

#ifndef __ppcdev_def_h_included_
#define __ppcdev_def_h_included_

#include <linux/interrupt.h>

#include "net_event.h"


// マクロ定義
#define PPCDEV_PAGESIZE      PAGE_SIZE                   // ページサイズ[Byte]
#define PPCDEV_MEMBLK        NET_PPC_DRAM_MBLK_SIZE      // DRAMメモリブロックサイズ[Byte]
#define PPCDEV_MEMSET        NET_PPC_MBLK_NUM            // DRAMメモリブロックセット数
#define PPCDEV_TXMSGW        NET_PPC_SRAM_MBLK_SIZE      // SRAM送信メッセージサイズ[Byte]
#define PPCDEV_KAPI_SYNC     0                           // カーネルAPI同期・非同期モード選択 0:ノンブロッキング -1:ブロッキング
#define PPCDEV_SRAM_BASE     NET_PPC_SRAM_BASE_PADDR     // 共有SRAMベースアドレス
#define PPCDEV_SRAM_SIZE     NET_PPC_SRAM_AREA_SIZE      // 共有SRAMサイズ
#define PPCDEV_TXSRAM_OFFSET NET_PPC_TX_SRAM_POFS        // 共有SRAMオフセットアドレス(送信)
#define PPCDEV_RXSRAM_OFFSET NET_PPC_RX_SRAM_POFS        // 共有SRAMオフセットアドレス(受信)
#define PPCDEV_TXDRAM_BASE   NET_PPC_TX_DRAM_PADDR       // 共有DRAMベースアドレス(送信)
#define PPCDEV_TXDRAM_SIZE   NET_PPC_DRAM_EACH_AREA_SIZE // 共有DRAMサイズ(送信)
#define PPCDEV_RXDRAM_BASE   NET_PPC_RX_DRAM_PADDR       // 共有DRAMベースアドレス(受信)
#define PPCDEV_RXDRAM_SIZE   NET_PPC_DRAM_EACH_AREA_SIZE // 共有DRAMサイズ(受信)
#define PPCDEV_MAXBUF        NET_PPC_SRAM_EACH_AREA_SIZE // SRAMデータ構造体 8[Byte] * 16[エントリ]
#define PPCDEV_DRAM_BASE     NET_PPC_DRAM_BASE_PADDR     // 共有DRAMベースアドレス
#define PPCDEV_DRAM_SIZE     NET_PPC_DRAM_AREA_SIZE      // 共有DRAMサイズ
#define PPCDEV_TXDRAM_OFFSET NET_PPC_TX_DRAM_POFS        // 共有DRAMオフセットアドレス(送信)
#define PPCDEV_RXDRAM_OFFSET NET_PPC_RX_DRAM_POFS        // 共有DRAMオフセットアドレス(受信)


// 構造体/共用体/型定義
typedef struct ppcdev_msgh_t {  // SRAMメッセージ構造体
    unsigned short id;          // ID
    unsigned short size;        // DRAMバッファサイズ
    unsigned long  txdadr;      // DRAMバッファアドレス
} ppcdev_msgh_t;
typedef struct ppcdev_memh_t {  // DRAMヘッダー構造体
    unsigned long signature;    // 0xBF FIX
    unsigned long en;           // 有効フラグ
    unsigned long size;         // バッファサイズ
    unsigned long next;         // メモリブロックポインタ
    unsigned long reserved;     // 未使用(アライメント調整)
} ppcdev_memh_t;
typedef struct ppcdev_que_t {   // 送受信キュー構造体
    char msgq[ PPCDEV_MAXBUF ]; // 送受信メッセージキュー
    int  id;                    // 送受信メッセージID
    int  rp;                    // 送受信キューリードポインタ
    int  wp;                    // 送受信キューライトポインタ
    char empty;                 // 送受信キューエンプティフラグ
    char full;                  // 送受信キューフルフラグ
    spinlock_t que_spinlock;    // 送受信キュースピンロック
    wait_queue_head_t write_q;  // 送受信キューライトウェイト
    wait_queue_head_t read_q;   // 送受信キューリードウェイト
} ppcdev_que_t;
typedef struct ppcdev_work_t {  // 拡張ワーク構造体
    struct work_struct work;    // ワーク構造体
    int qid;                    // キューID
} ppcdev_work_t;
typedef irqreturn_t ( *ppcdev_irq_handler_t )( int, int, const void *, size_t ); // コールバック
enum ppcdev_rxresult { _SUCCESS = 1, _RETRY, _FATAL };    // 受信結果
enum ppcdev_queueid { _TX = 0, _NWA, _ETH, _DMA, _MAXQ }; // 送受信キュー識別子
enum ppcdev_bool { _FALSE = 0, _TRUE };

#endif /* __ppcdev_def_h_included_ */
