/**
 *  Copyright (C) 2012 Panasonic Corporation. All Rights Reserved.
 *
 * @file
 * @brief   CPU間通信D/Dメイン処理ヘッダー
 */

#ifndef __ppcdev_main_h_included_
#define __ppcdev_main_h_included_

#include <linux/mutex.h>
#include <linux/spinlock.h>
#include <linux/kthread.h>
#include <linux/workqueue.h>
#include "ppcdev_def.h"

// グローバル変数
extern void __iomem *srbase_ioaddr;             // SRAMベースアドレス
extern void __iomem *txsram_ioaddr;             // 送信SRAMアドレス
extern void __iomem *rxsram_ioaddr;             // 受信SRAMアドレス
extern void __iomem *drbase_ioaddr;             // DRAMベースアドレス
extern void __iomem *txdram_ioaddr;             // 送信DRAMアドレス
extern void __iomem *rxdram_ioaddr;             // 受信DRAMアドレス
extern int ppcirq_fd;                           // PPC割り込みドライバディスクリプタ
extern ppcdev_work_t ppcdev_work[ _MAXQ ];      // ワークキュー配列
extern ppcdev_irq_handler_t ppcdev_cb[ _MAXQ ]; // コールバック配列
extern spinlock_t ppcdev_qptr_spinlock;         // 送受信キュー構造体配列サイズ
extern struct task_struct *ppcdev_kth;          // カーネルスレッド
extern int ppcdev_retry;                        // 送信リトライ回数 デフォルト 1回
extern struct mutex ppcdev_tx_mutex;            // NWA/ETH/DMA送信ミューテックス
extern struct mutex ppcdev_kmalloc_mutex;       // kmallocミューテックス
extern struct mutex ppcdev_kfree_mutex;         // kfreeミューテックス
extern struct mutex ppcdev_retry_mutex;         // リトライミューテックス


// プロトタイプ宣言
extern int __ppcdev_open( int qid, int id );
extern int __ppcdev_release( int qid );
extern ssize_t __ppcdev_read( int qid, char *buf, size_t count, int tmout );
extern ssize_t __ppcdev_write( int qid, const char *buf, size_t count, int tmout );
extern void *__ppcdev_kmalloc( size_t size );
extern void __ppcdev_kfree( const void *buf );

#endif /* __ppcdev_main_h_included_ */
