/**
 *  Copyright (C) 2012 Panasonic Corporation. All Rights Reserved.
 *
 * @file
 * @brief   CPU間通信割り込みD/Dヘッダー
 */
#ifndef __ppcirq_h_included_
#define __ppcirq_h_included_

#include <linux/interrupt.h>

// マクロ定義

// 構造体/共用体/型定義
// SW割り込みオフセットアドレス
enum ppcirq_ircofs_t {
    PPC_IRC0DINT0_OFS = 0xA14,
    PPC_IRC0DINT1_OFS = 0xA18,
    PPC_IRC1DINT0_OFS = 0xC14,
    PPC_IRC1DINT1_OFS = 0xC18,
    PPC_IRC2DINT0_OFS = 0xE14,
    PPC_IRC2DINT1_OFS = 0xE18
};
// 割り込みマスクビット割り当て
enum ppcirq_mask_t {
    PPC_IRC0DINT0_MASK = 0x00000001,
    PPC_IRC0DINT1_MASK = 0x00000002,
    PPC_IRC1DINT0_MASK = 0x00000004,
    PPC_IRC1DINT1_MASK = 0x00000008,
    PPC_IRC2DINT0_MASK = 0x00000010,
    PPC_IRC2DINT1_MASK = 0x00000020
};
// 割り込み優先順位(降順)
enum ppcirq_pil_t {
    PPCIRQ_PIL_KAO = 0,
    PPCIRQ_PIL_NWA,
    PPCIRQ_PIL_END // 優先順位付けする割り込みの総数
};
// 受信割り込みコールバック
typedef irqreturn_t ( *ppcirq_rxcb_t )( int, void * );
// 送信割り込みコールバック
typedef void ( *ppcirq_txcb_t )( void * );

// プロトタイプ宣言
int ppcirq_kopen( int priority, int mask, int rxofs, int txofs );
int ppcirq_kclose( int fd );
int ppcirq_kenable( int fd );
int ppcirq_kdisable( int fd );
int ppcirq_ksetcb( int fd, ppcirq_rxcb_t rxcb, ppcirq_txcb_t txcb );
int ppcirq_kdelcb( int fd );
int ppcirq_ksend( int fd );
int ppcirq_ksend_na( int fd );

#endif /* __ppcirq_h_included_ */
