
#include <linux/proc_fs.h>

#include "iccdrv.h"

#ifdef CONFIG_PROC_FS

static struct proc_dir_entry *icc_proc_entry;

static int icc_read_proc(char *page, char **start, off_t off,
			 int count, int *eof, void *data)
{
	char *p = page;
	int len;
	int i;

	p += sprintf(p, "icc_read_proc\n");

	/* need to lock? ... */
	for (i = 0; iccdrv_iomap[i].size > 0; i++) {
		p += sprintf(p, "ICC map area: P:%08lx->V:%08lx (%08x byte)\n",
			     iccdrv_iomap[i].paddr, iccdrv_iomap[i].vaddr,
			     iccdrv_iomap[i].size);
	}
	len = p - page - off;
	if (len < count) {
		*eof = 1;
		if (len <= 0) return 0;
	} else {
		len = count;
	}
	*start = page + off;
	return len;
}

static int icc_write_proc(struct file *file, const char __user *buffer,
			  unsigned long count, void *data)
{
	return count;
}

int icc_init_proc(void)
{
	int ret = 0;
	icc_proc_entry = create_proc_entry("driver/icc", 0644, NULL);
	if (!icc_proc_entry) {
		ret = -ENOMEM;
		goto out;
	}
	icc_proc_entry->data = NULL;
	icc_proc_entry->read_proc = icc_read_proc;
	icc_proc_entry->write_proc = icc_write_proc;
out:
	return ret;
}

int icc_exit_proc(void)
{
	if (icc_proc_entry) {
		remove_proc_entry("driver/icc", NULL);
		icc_proc_entry = NULL;
	}
	return 0;
}

#endif
