#ifndef _ICCDRV_PVC02V_H_
#define _ICCDRV_PVC02V_H_

#include "icc_cfg.h"

#undef ICC_USE_DMB

#define ICC_DMB_WAIT_USEC	25
#define ICC_CPU_HZ		498000000
#define ICC_DMB_WAIT_NUM	(ICC_CPU_HZ/1000/1000 * ICC_DMB_WAIT_USEC)

#if !defined(ICC_USE_DMB)
#if defined(__CC_ARM)
static __inline void icc_dmb_wait(void)
#elif defined(__GNUC__)
static inline void icc_dmb_wait(void)
#endif
{
	int i;
	for (i = 0; i < ICC_DMB_WAIT_NUM/10; i++) {
#if defined(__CC_ARM)
		__nop();
		__nop();
		__nop();
		__nop();
		__nop();
		__nop();
		__nop();
		__nop();
		__nop();
		__nop();
#elif defined(__GNUC__)
		__asm__ __volatile__(
			"nop	\n"
			"nop	\n"
			"nop	\n"
			"nop	\n"
			"nop	\n"
			"nop	\n"
			"nop	\n"
			"nop	\n"
			"nop	\n"
			"nop	\n"
			);
#endif
	}
}

static inline void	arm_dmb(void)
{
	asm volatile(
		// r0:荞ݐݒ
		// r1:DCache ON̐ݒ
		// r2:DCache OFF̐ݒ
		"push	{r0-r2}					\n"	
		"mrs	r0, cpsr				\n"	// 荞݋/֎~ۑ
		"cpsid	i						\n"	// 荞݋֎~
		"mrc 	p15, 0, r1, c1, c0, 0	\n"
		"bic 	r2, r1, #4				\n"
		"mcr 	p15, 0, r2, c1, c0, 0	\n"	// DCache OFF
		"mcr 	p15, 0, r1, c1, c0, 0	\n"	// DCache ON(ɖ߂)
		"msr 	cpsr_c, r0				\n" // 荞݋/֎~ɖ߂
		"pop	{r0-r2}					\n"	
	);
}

#endif /* !ICC_USE_DMB */

#if defined(_PGR)  /* dummy for PGRelief */
#define memory_barrier()
#else /* _PGR */
#if defined(__GNUC__)
#if defined(ICC_USE_DMB)
#define memory_barrier()	__asm__ __volatile__("dmb": : : "memory")
#else /* !ICC_USE_DMB */
#define memory_barrier()	arm_dmb()
#endif /* !ICC_USE_DMB */
#elif defined(__CC_ARM)
#if defined(ICC_USE_DMB)
#define memory_barrier()	__asm {dmb}
#else /* !ICC_USE_DMB */
#define memory_barrier()	arm_dmb()
#endif /* !ICC_USE_DMB */
#endif
#endif /* _PGR */

#if defined(_PGR)  /* dummy for PGRelief */
static inline unsigned long icc_local_irqsave(void) { return 0; }
#else /* _PGR */
static inline unsigned long icc_local_irqsave(void)
{
	unsigned long flags = 0;
#if defined(__GNUC__)
	__asm__ __volatile__(
		"	mrs	%0, cpsr\n"
		"	cpsid	i	\n"
		: "=r"(flags)
		:
		: "memory", "cc"
		);
#elif defined(__CC_ARM)
	__asm {
		mrs	flags, cpsr;
		cpsid	i;
	}
#endif
	memory_barrier();
	return flags;
}
#endif /* _PGR */

#if defined(_PGR)  /* dummy for PGRelief */
static inline int icc_local_irqrestore(unsigned long flags) { return 0; }
#else /* _PGR */
static inline int icc_local_irqrestore(unsigned long flags)
{
	memory_barrier();
#if defined(__GNUC__)
	__asm__ __volatile__(
		"	msr	cpsr_c, %0\n"
		:
		: "r" (flags)
		: "memory", "cc"
		);
#elif defined(__CC_ARM)
	__asm {
		msr	cpsr_c, flags;
	}
#endif
	return 0;
}
#endif /* _PGR */

#if defined(_PGR)  /* dummy for PGRelief */
static inline int icc_cpu_self(void) { return 0; }
#else /* _PGR */
static inline int icc_cpu_self(void)
{
	unsigned int mpidr = 0;
#if defined(__GNUC__)
	__asm__ __volatile__(
		"mrc	p15, 0, %0, c0, c0, 5\n"
		: "=r"(mpidr)
		:
		: "memory", "cc"
		);
#elif defined(__CC_ARM)
	__asm {
		mrc	p15, 0, mpidr, c0, c0, 5;
	}
#endif

	mpidr &= 0x00000003;

	return mpidr;
}
#endif /* _PGR */

#if defined(ICC_LINUX)
#define ICC_DRIVER_NUM	1
#define current_iccdrv()	(&iccdrv_info[0])
#else /* ICC_ITRON || ICC_OSLESS */

#ifdef ICC_ON_CPU0
extern struct iccdrv_info_struct iccdrv_info_0;
#endif
#ifdef ICC_ON_CPU1
extern struct iccdrv_info_struct iccdrv_info_1;
#endif
#ifdef ICC_ON_CPU2
extern struct iccdrv_info_struct iccdrv_info_2;
#endif
#ifdef ICC_ON_CPU3
extern struct iccdrv_info_struct iccdrv_info_3;
#endif

static inline struct iccdrv_info_struct *current_iccdrv(void)
{
	int cpu = icc_cpu_self();
	switch (cpu) {
#ifdef ICC_ON_CPU0
	case 0:
		return &iccdrv_info_0;
#endif
#ifdef ICC_ON_CPU1
	case 1:
		return &iccdrv_info_1;
#endif
#ifdef ICC_ON_CPU2
	case 2:
		return &iccdrv_info_2;
#endif
#ifdef ICC_ON_CPU3
	case 3:
		return &iccdrv_info_3;
#endif
	default:
		break;
	}
	return NULL;
}

#endif /* ICC_ITRON || ICC_OSLESS */

/* context check */
#if defined(ICC_ITRON)
#include <kernel.h>
static inline int icc_context_check(void)
{
	return (sns_ctx() | sns_dsp() | sns_loc());
}
#elif defined(ICC_OSLESS)
static inline int icc_context_check(void) { return 0; }
#endif

static inline void *__iccdrv_gic_io(unsigned long addr)
{
	int current_cpu = icc_cpu_self();
	return __iccdrv_io(addr + (current_cpu * GIC_PERCPU_OFFSET));
}

#if !defined(__ICC_PVC02V_V2P__) /* V9 */
static inline unsigned long gic_cpu_map(int cpu)
{
	return 1 << cpu;
}
#else
static inline unsigned long gic_cpu_map(int cpu)
{
	return 1 << (cpu + 2);
}
#endif

/* raise interrupt to other CPU */
static inline int icc_raise_irq(uint16_t id)
{
	int cpu = icc_id_owner(id);
	unsigned long map = gic_cpu_map(cpu);
	unsigned long gic_dist_sgir = GIC_DIST_BASE + GIC_DIST_SGIR;
	unsigned long send_irq[] = \
		{ICC_CPU0_IRQ, ICC_CPU1_IRQ, ICC_CPU2_IRQ, ICC_CPU3_IRQ};

	if (cpu < 0 || cpu >= 4) {
		return cpu;
	}

	*(volatile unsigned long *)__iccdrv_gic_io(gic_dist_sgir) = \
		(send_irq[cpu] | map << 16);
	return 0;
}

/* raise interrupt to self CPU */
static inline int icc_raise_irq_self(void)
{
	int cpu = icc_cpu_self();
	unsigned long map = gic_cpu_map(cpu);
	unsigned long gic_dist_sgir = GIC_DIST_BASE + GIC_DIST_SGIR;
	unsigned long send_irq[] = \
		{ICC_CPU0_IRQ, ICC_CPU1_IRQ, ICC_CPU2_IRQ, ICC_CPU3_IRQ};

	*(volatile unsigned long *)__iccdrv_gic_io(gic_dist_sgir) = \
		(send_irq[cpu] | map << 16);
	return 0;
}

static inline void icc_intc_clear(int irq)
{
	unsigned long gic_dist_cpendsgir = GIC_DIST_BASE + GIC_DIST_CPENDSGIR + irq;

	/* SGI Pending clear */
	*(volatile unsigned char*)__iccdrv_gic_io(gic_dist_cpendsgir) =  0xff;
}

static inline void icc_intc_enable(int irq)
{
	unsigned long gic_dist_iser = GIC_DIST_BASE + GIC_DIST_ISER + ((irq / 32) * 4);

	*(volatile unsigned long *)__iccdrv_gic_io(gic_dist_iser) = 1 << (irq % 32);
}

/* clear interrupts to self CPU */
static inline void icc_clear_irq(void)
{
	icc_intc_clear(ICC_RCV_IRQ);
}

/* enable interrupts to self CPU */
static inline void icc_enable_irq(void)
{
	icc_intc_enable(ICC_RCV_IRQ);
}

static inline icc_fifocb_t *icc_fifocb_addr(int from, int to)
{
	switch(from) {
	case ICC_CPU0:
		switch (to) {
		case ICC_CPU1:
			return (icc_fifocb_t *)__iccdrv_io(CPU0_1_FIFO_ADDR);
		case ICC_CPU2:
			return (icc_fifocb_t *)__iccdrv_io(CPU0_2_FIFO_ADDR);
		case ICC_CPU3:
			return (icc_fifocb_t *)__iccdrv_io(CPU0_3_FIFO_ADDR);
		default:
			break;
		}
		break;
	case ICC_CPU1:
		switch (to) {
		case ICC_CPU0:
			return (icc_fifocb_t *)__iccdrv_io(CPU1_0_FIFO_ADDR);
		case ICC_CPU2:
			return (icc_fifocb_t *)__iccdrv_io(CPU1_2_FIFO_ADDR);
		case ICC_CPU3:
			return (icc_fifocb_t *)__iccdrv_io(CPU1_3_FIFO_ADDR);
		default:
			break;
		}
		break;
	case ICC_CPU2:
		switch (to) {
		case ICC_CPU0:
			return (icc_fifocb_t *)__iccdrv_io(CPU2_0_FIFO_ADDR);
		case ICC_CPU1:
			return (icc_fifocb_t *)__iccdrv_io(CPU2_1_FIFO_ADDR);
		case ICC_CPU3:
			return (icc_fifocb_t *)__iccdrv_io(CPU2_3_FIFO_ADDR);
		default:
			break;
		}
		break;
	case ICC_CPU3:
		switch (to) {
		case ICC_CPU0:
			return (icc_fifocb_t *)__iccdrv_io(CPU3_0_FIFO_ADDR);
		case ICC_CPU1:
			return (icc_fifocb_t *)__iccdrv_io(CPU3_1_FIFO_ADDR);
		case ICC_CPU2:
			return (icc_fifocb_t *)__iccdrv_io(CPU3_2_FIFO_ADDR);
		default:
			break;
		}
		break;
	default:
		break;
	}
	return NULL;
}

/* get receive fifo address */
static inline icc_fifocb_t *icc_get_receivefifo(int from_cpu)
{
	int current_cpu = icc_cpu_self();

	return icc_fifocb_addr(from_cpu, current_cpu);
}

/* get send fifo address */
static inline icc_fifocb_t* icc_get_sendfifo(int to_cpu)
{
	int current_cpu = icc_cpu_self();

	return icc_fifocb_addr(current_cpu, to_cpu);
}

#endif /* _ICCDRV_PVC02V_H_ */
