
#ifndef _ICSMDRV_LINUX_H_
#define _ICSMDRV_LINUX_H_

#include <linux/cdev.h>
#include <linux/string.h>
#include <linux/jiffies.h>
#include <linux/sched.h>
#include <asm/uaccess.h>

#include "icsm_cfg.h"

typedef struct icsmdrv_info_struct {
	int timeout[ICSM_MEMBLK_MAX];
} icsmdrv_info_t;

struct icsmdrv_iomap_struct {
	unsigned long vaddr;	/* virtual (mapping) address */
	unsigned long paddr;	/* physical address  */
	int size;		/* size */
};

typedef unsigned long icsm_time_t;

static inline icsm_time_t msecs_to_icsm_time(unsigned int msec)
{
	return msecs_to_jiffies(msec);
}

static inline unsigned int icsm_time_to_msecs(icsm_time_t time)
{
	return jiffies_to_msecs(time);
}

static inline int icsm_sleep(icsm_time_t time)
{
	set_current_state(TASK_INTERRUPTIBLE);
	schedule_timeout(time);
	return 0;
}

extern struct icsmdrv_iomap_struct icsmdrv_iomap[];
static inline void __iomem *__icsmdrv_io(unsigned long addr)
{
	int i;

	for (i = 0; icsmdrv_iomap[i].size > 0; i++) {
		if (icsmdrv_iomap[i].paddr <= addr &&
		    addr < icsmdrv_iomap[i].paddr + icsmdrv_iomap[i].size) {
			return (void __iomem*)(addr - icsmdrv_iomap[i].paddr \
					       + icsmdrv_iomap[i].vaddr);
		}
	}

	printk(KERN_ERR "__icsmdrv_io: Unmapped address(%08lx)\n", addr);
	BUG();
}

#endif /* _ICSMDRV_LINUX_H_ */
