/*
 * Copyright (C) 2011 Panasonic Corporation
 * All Rights Reserved.
 *
 */
#ifndef __LINKED_LIST_H_
#define __LINKED_LIST_H_

#if defined(__CC__)
#ifndef inline
#define inline
#endif
#elif defined(__CC_ARM)
#ifndef inline
#define inline __inline
#endif
#endif

struct linked_list {
	struct linked_list *next;
	struct linked_list *prev;
};

/* list header file */
#define CREATE_LIST_HEAD(head)  struct linked_list head = {&(head), &(head)}

static inline int is_list_empty(struct linked_list *list)
{
	return (list->next == list);
}

static inline void init_list(struct linked_list *list)
{
	list->next = list;
	list->prev = list;
}

static inline void add_list(struct linked_list *head, struct linked_list *list)
{
	struct linked_list *last = head->prev;

	list->next = head;
	list->prev = last;
	head->prev = list;
	last->next = list;
}

static inline void add_list_top(struct linked_list *head, struct linked_list *list)
{
    struct linked_list *last = head->next;

    list->next = last;
    list->prev = head;
    head->next = list;
    last->prev = list;
}
static inline void remove_list(struct linked_list *list)
{
	struct linked_list *next = list->next;
	struct linked_list *prev = list->prev;

	prev->next = next;
	next->prev = prev;
	init_list(list);
}

#define for_each_list(__head, __list, __next)			\
	for (__list = (__head)->next, __next = (__list)->next;	\
	     (__list) != (__head);				\
	     __list = (__next), __next = (__list)->next)

#endif /* __LINKED_LIST_H_ */
