/*
 *  sveth_debug.h
 *
 *	The virtual ethernet driver for the SPiCa movie system.
 *
 *  Copyright (C) 2012 Panasonic Corporation. All Rights Reserved.
 *
 */

#ifndef	_SVETH_DEBUG_H_
#define	_SVETH_DEBUG_H_

#include <itron.h>
#include <kernel.h>


#define _unused(_v)				(void)(_v)



/**
 *	definition for the kernel
 */
#if defined(__KERNEL__)

/* assrt */
#define assert(expr) \
	do { \
		if(unlikely(!(expr))) { \
			pr_err("Assertion failed! condition: (%s), %s, %s, line=%d\n", \
					#expr, __FILE__, __func__, __LINE__); \
		} \
	} while(0)

#else	/* __KERNEL__ */

/* assert */
#include <assert.h>

/* printk */
#define KERN_EMERG						"[printk: EMERG  ] "
#define KERN_ALERT						"[printk: ALERT  ] "
#define KERN_CRIT						"[printk: CRIT   ] "
#define KERN_ERR						"[printk: ERR    ] "
#define KERN_WARNING					"[printk: WARNING] "
#define KERN_NOTICE						"[printk: NOTICE ] "
#define KERN_INFO						"[printk: INFO   ] "
#define KERN_DEBUG						"[printk: DEBUG  ] "

#define printk(fmt, ...)				fprintf(stderr, fmt, ##__VA_ARGS__)

#endif	/* __KERNEL__ */



/* message */
#define SVETH_PRINT(lv, fmt, ...)		\
	do {	\
		printk(lv	"[%s: %s()] " fmt, devname_p, __func__, ##__VA_ARGS__);	\
	} while(0)

#if defined(CONFIG_SVETH_DEBUG)

/* debug message */
#define DPRINT(fmt, ...)				printk(fmt, ##__VA_ARGS__)

#define PEMERG(fmt, ...)				SVETH_PRINT(KERN_EMERG,		fmt, ##__VA_ARGS__)
#define PALERT(fmt, ...)				SVETH_PRINT(KERN_ALERT,		fmt, ##__VA_ARGS__)
#define PCRIT(fmt, ...)					SVETH_PRINT(KERN_CRIT,		fmt, ##__VA_ARGS__)
#define PERR(fmt, ...)					SVETH_PRINT(KERN_ERR,		fmt, ##__VA_ARGS__)
#define PWARNING(fmt, ...)				SVETH_PRINT(KERN_WARNING,	fmt, ##__VA_ARGS__)
#define PNOTICE(fmt, ...)				SVETH_PRINT(KERN_NOTICE,	fmt, ##__VA_ARGS__)
#define PINFO(fmt, ...)					SVETH_PRINT(KERN_INFO,		fmt, ##__VA_ARGS__)
#define PDEBUG(fmt, ...)				SVETH_PRINT(KERN_DEBUG,		fmt, ##__VA_ARGS__)

#define PERRMSG(expr, ...)				\
	do {	\
		if(unlikely(!(expr))) {	\
			PNOTICE("(%s) fail", #expr, ##__VA_ARGS__);	\
		}	\
	} while(0)


#else	/* CONFIG_SVETH_DEBUG */

/* debug message */
#define DPRINT(fmt, ...)				do {} while(0)	/* do nothing */
#define PEMERG(fmt, ...)				do {} while(0)	/* do nothing */
#define PALERT(fmt, ...)				do {} while(0)	/* do nothing */
#define PCRIT(fmt, ...)					do {} while(0)	/* do nothing */
#define PERR(fmt, ...)					do {} while(0)	/* do nothing */
#define PWARNING(fmt, ...)				do {} while(0)	/* do nothing */
#define PNOTICE(fmt, ...)				do {} while(0)	/* do nothing */
#define PINFO(fmt, ...)					do {} while(0)	/* do nothing */
#define PDEBUG(fmt, ...)				do {} while(0)	/* do nothing */

#define PERRMSG(expr, ...)				do {} while(0)	/* do nothing */

#endif	/* CONFIG_SVETH_DEBUG */


#if defined(CONFIG_SVETH_ASSERT)

/* assert */
#define sveth_assert(expr)				assert(expr)

#else	/* CONFIG_SVETH_ASSERT */

#define sveth_assert(expr)				do {} while(0)	/* do nothing */

#endif	/* CONFIG_SVETH_ASSERT */


#endif	/* _SVETH_DEBUG_H_ */


