/*
 *  sveth_debug.h
 *
 *	The virtual ethernet driver for the SPiCa movie system.
 *
 *  Copyright (C) 2012 Panasonic Corporation. All Rights Reserved.
 *
 */

#ifndef	_SVETH_DEBUG_UTIL_H_
#define	_SVETH_DEBUG_UTIL_H_

#include "sveth_debug.h"
#include "net_event.h"


#if defined(CONFIG_SVETH_DEBUG)


#define _dump_tx_ack(p, flg)		\
	do {	\
		if(likely((flg) == 1)) {	\
			PNOTICE("tx_ack");	\
			PNOTICE("    + pkt_id=0x%08lX",	(p)->pkt_id);	\
			PNOTICE("    + errcode=%d",		(p)->errcode);	\
		}	\
	} while(0)


#define _dump_rx_pkt(p, flg)		\
	do {	\
		if(likely((flg) == 1)) {	\
			PNOTICE("rx_pkt");	\
			PNOTICE("    + pkt_id=0x%08lX",	(p)->pkt_id);	\
			PNOTICE("    + len=%d",			(p)->len);	\
			PNOTICE("    + errcode=%d",		(p)->errcode);	\
			PNOTICE("    + addr_p=%p",		(p)->addr_p);	\
		}	\
	} while(0)



#define NOL				(16)
#define _SPRITTER(i) \
	( \
		((i) == 0)			?			"      %06X: " : \
		((i) % (NOL) == 0)	?	"\n"	"      %06X: " : \
										" " \
	)

static inline int _dump_buffer(unsigned char* p, int len) {
	int i;
	
	DPRINT("      offset: 00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F\n");
	DPRINT("    ---------------------------------------------------------\n");

	for(i = 0; i < len; i++) {
		DPRINT(_SPRITTER(i), i / NOL);
		DPRINT("%02X", p[i]);
	}
	DPRINT("\n");
	DPRINT("    ---------------------------------------------------------\n");

	return 0;
}

static inline int _dump_nemsg(const NET_EVT_MSG* msg_p) {
	sveth_ipcc_pkt_t* pkt_p;
	uint16_t n;
	unsigned char* tmp_p;
	
	sveth_assert(msg_p);

	pkt_p = (sveth_ipcc_pkt_t*)&msg_p->Data.SvethIpccPkt;

	/** [TEST] *** unsafe!! *** */
	tmp_p = (void*)msg_p;
	
	
	n = (uint16_t)offsetof(NET_EVT_MSG, Data);		/* net_evt_msg header length */
	n += (uint16_t)(
		/* cmd size (with padding) */
		pkt_p->head.len -
		/* padding size */
		(offsetof(struct sveth_ipcc_pkt_t, cmd) - sizeof(pkt_p->head)) +
		/* header size */
		sizeof(pkt_p->head)
	);												/* sveth_cmd_pkt length */
	
	DPRINT("[DUMP: NET_EVT_MSG] -----------------------------------------\n");
	DPRINT("  msg_p=%p, pkt_p=%p\n", msg_p, pkt_p);
	DPRINT("  [pkt_p->head]\n");
	DPRINT("    start_id=0x%04X, cmd_id=0x%02X, len=%d\n",
					pkt_p->head.start_id, pkt_p->head.cmd_id, pkt_p->head.len);
	_dump_buffer(tmp_p, n);
	DPRINT("\n\n");

	return 0;
}


/* dump data */
#define DUMP_BUFFER(p, len, flg)				\
	do {	\
		if(unlikely(!!(flg))) {	\
			DPRINT("[DUMP: BUFFER] ----------------------------------------------\n");	\
			_dump_buffer((p), (len));	\
		}	\
	} while(0)


/* dump data */
#define DUMP_NEMSG(p, flg)				\
	do {	\
		if(unlikely(!!(flg))) {	\
			_dump_nemsg((p));	\
		}	\
	} while(0)


#else	/* CONFIG_SVETH_DEBUG */

#define _dump_tx_ack(p, flg)			/* nop */
#define _dump_rx_pkt(p, flg)			/* nop */
#define DUMP_BUFFER(p, len, flg)		/* nop */
#define DUMP_NEMSG(msg_p, flg)			/* nop */

#endif	/* CONFIG_SVETH_DEBUG */


#endif	/* _SVETH_DEBUG_UTIL_H_ */


