/*
 *  sveth_ipcc_kapi.h
 *
 *	The virtual ethernet driver for the SPiCa movie system.
 *
 *  Copyright (C) 2012 Panasonic Corporation. All Rights Reserved.
 *
 */

#ifndef	_SVETH_IPCC_KAPI_H_
#define	_SVETH_IPCC_KAPI_H_

#include "sveth_ipcc_def.h"


/**
 *	event operation
 */

typedef struct sveth_ipcc_event_t {
	
	/* SVETH_IPCC_INIT_ACK */
	int (*pf_init_ack) (int fd, const void* param_p, unsigned short errcode, volatile sveth_ipcc_vreg_t* vreg_p, unsigned long vreg_size);
	
	/* SVETH_IPCC_QUIT_ACK */
	int (*pf_quit_ack) (int fd, const void* param_p, unsigned short errcode);
	
	/* SVETH_IPCC_RECV */
	int (*pf_recv_pkt) (int fd, const void* param_p, sveth_ipcc_rtd_t* rtd_p);
	
	/* SVETH_IPCC_STATIND */
	int (*pf_stat_ind) (int fd, const void* param_p, unsigned char prm);
	
	/* SVETH_IPCC_GETPRM_ACK(SVETH_IPCC_GETPRM_SUB_INTR) */
	int (*pf_get_int_stat) (int fd, const void* param_p, unsigned short errcode, unsigned short req_id, unsigned char flg);
	
	/* SVETH_IPCC_GETPRM_ACK(SVETH_IPCC_GETPRM_SUB_MAC) */
	int (*pf_get_macaddr) (int fd, const void* param_p, unsigned short errcode, unsigned short req_id, unsigned char* macaddr_p);
	
	/* SVETH_IPCC_GETPRM_ACK(SVETH_IPCC_GETPRM_SUB_LINK) */
	int (*pf_get_link_stat) (int fd, const void* param_p, unsigned short errcode, unsigned short req_id, unsigned char speed, unsigned char duplex);
	
} sveth_ipcc_event_t;



/**
 *	IPC API [debug]
 */

void sveth_ipcc_dump_priv_info(int fd);



/**
 *	IPC API [dd]
 */

/* ipcc[dd]: init, exit, open, close */
int __init sveth_ipcc_dd_init(void);
void __exit sveth_ipcc_dd_exit(void);
int sveth_ipcc_dd_open(int id, const sveth_ipcc_event_t* event_p, const void* param_p);
int sveth_ipcc_dd_close(int fd);



/**
 *	IPC API [kapi]
 */

/* SVETH_IPCC_INIT */
int sveth_ipcc_kapi_init(int fd);

/* SVETH_IPCC_QUIT */
int sveth_ipcc_kapi_quit(int fd);

/* SVETH_IPCC_SEND */
int sveth_ipcc_kapi_send_packet(int fd, const sveth_ipcc_rtd_t* rtd_p);

/* SVETH_IPCC_SETPRM(SVETH_IPCC_SETPRM_SUB_INTR) */
int sveth_ipcc_kapi_set_interrupt_mode(int fd, unsigned char flg);

/* SVETH_IPCC_SETPRM(SVETH_IPCC_SETPRM_SUB_MAC) */
int sveth_ipcc_kapi_set_mac_address(int fd, const unsigned char* macaddr_p);

/* SVETH_IPCC_SETPRM(SVETH_IPCC_SETPRM_SUB_POWER) */
int sveth_ipcc_kapi_set_power_mode(int fd, unsigned char mode);

/* SVETH_IPCC_SETPRM(SVETH_IPCC_SETPRM_SUB_MCAST) */
int sveth_ipcc_kapi_set_muticast_mode(int fd, unsigned char flg);

/* SVETH_IPCC_SETPRM(SVETH_IPCC_SETPRM_SUB_WUP) */
int sveth_ipcc_kapi_set_wol_mode(int fd, unsigned char flg);

/* SVETH_IPCC_GETPRM(SVETH_IPCC_GETPRM_SUB_INTR) */
int sveth_ipcc_kapi_request_interrupt_state(int fd, unsigned short req_id);

/* SVETH_IPCC_GETPRM(SVETH_IPCC_GETPRM_SUB_MAC) */
int sveth_ipcc_kapi_request_mac_address(int fd, unsigned short req_id);

/* SVETH_IPCC_GETPRM(SVETH_IPCC_GETPRM_SUB_LINK) */
int sveth_ipcc_kapi_request_link_state(int fd, unsigned short req_id);

/* SVETH_IPCC_STATIND */
int sveth_ipcc_kapi_notify_status(int fd, unsigned char prm);


#endif	/* _SVETH_IPCC_KAPI_H_ */


