/*
 *  sveth_ipcc_local.h
 *
 *	The virtual ethernet driver for the SPiCa movie system.
 *
 *  Copyright (C) 2012 Panasonic Corporation. All Rights Reserved.
 *
 */

#ifndef	_SVETH_IPCC_LOCAL_H_
#define	_SVETH_IPCC_LOCAL_H_

#include <itron.h>
#include <kernel.h>

#include "sveth_debug.h"
#include "sveth_ipcc_def.h"
#include "sveth_ipcc_kapi.h"


#define SVETH_IPCC_LOCAL_NUM			(2)



/**
 *	[data types]
 */
typedef struct sveth_ipcc_local_t {
	int							used;
	
	int							fd;
	int							id;
	int							ppcdev_fd;
	sveth_ipcc_event_t			event;
	void*						param_p;
	
	spinlock_t					spinlock;
	unsigned long				flags;
	
} sveth_ipcc_local_t;


/**
 *	macros
 */

#define _sveth_lock_init(_p) \
			do { spin_lock_init(&(_p)->spinlock); } while(0)

#define _sveth_lock_irq(_p, _flags) \
			do { spin_lock_irqsave(&(_p)->spinlock, (_flags)); } while(0)

#define _sveth_unlock_irq(_p, _flags) \
			do { spin_unlock_irqrestore(&(_p)->spinlock, (_flags)); } while(0)


#define _sveth_ipcc_numof_array(_a) \
			(sizeof(_a) / sizeof(_a[0]))

#define _sveth_ipcc_foreach(_p, _a) \
			for((_p) = &(_a)[0]; (_p) < &(_a)[_sveth_ipcc_numof_array(_a)]; (_p)++)

#define _sveth_ipcc_fd_is_safety(_fd, _a) \
			((likely(0 <= (_fd)) && likely((_fd) < _sveth_ipcc_numof_array(_a))))

#define _sveth_ipcc_is_unused(_p) \
			likely((_p)->used != 1)

#define _sveth_ipcc_id_is_registerd(_p, _id)		unlikely((_p)->id == (_id))

#define _sveth_ipcc_get_id(_p)						((_p)->id)

#define _sveth_ipcc_get_ppcdev_fd(_p) \
			((_p)->ppcdev_fd)

#define _sveth_ipcc_pkt_size(p, len) \
			/** padding size + payload size */	\
			(offsetof(struct sveth_ipcc_pkt_t, cmd) - sizeof((p)->head)) + (len)

#define _sveth_ipcc_set_event(_dst_p, _src_p) \
			do { \
				*(_dst_p)			= *(_src_p); \
			} while(0)

#define _sveth_ipcc_init_event(_p) \
			do { \
				const static sveth_ipcc_event_t ___tmp = {0}; \
				_sveth_ipcc_set_event((_p), &___tmp); \
			} while(0)

#define _sveth_ipcc_init_local(_p) \
			do { \
				(_p)->used			= 0; \
				(_p)->fd			= -1; \
				(_p)->id			= -1; \
				(_p)->ppcdev_fd		= -1; \
				(_p)->param_p		= 0; \
				_sveth_ipcc_init_event(&(_p)->event); \
			} while(0)

#define _sveth_ipcc_exit_local(_p) \
			do { \
				(_p)->used			= 1; \
				(_p)->fd			= -1; \
				(_p)->id			= -1; \
				(_p)->ppcdev_fd		= -1; \
				(_p)->param_p		= 0; \
				_sveth_ipcc_init_event(&(_p)->event); \
			} while(0)

#define _sveth_ipcc_set_local(_p, _fd, _id, _ppcdev_fd, _event_p, _param_p) \
			do { \
				(_p)->used			= 1; \
				(_p)->fd			= (_fd); \
				(_p)->id			= (_id); \
				(_p)->ppcdev_fd		= (_ppcdev_fd); \
				(_p)->param_p		= (_param_p); \
				_sveth_ipcc_set_event(&(_p)->event, (_event_p)); \
			} while(0)


#define _sveth_ipcc_current_local(_fd, _a) \
			( \
				likely(_sveth_ipcc_fd_is_safety((_fd), (_a)) != 1) ? \
				0 : \
				likely((_a)[(_fd)].fd != (_fd)) ? \
				0 : &(_a)[(_fd)] \
			)



static inline sveth_ipcc_cmd_t* sveth_ipcc_get_cmd(const sveth_ipcc_pkt_t* pkt_p, unsigned char* cmd_id_p) {
	if(unlikely(pkt_p == 0) || unlikely(cmd_id_p == 0) || unlikely(pkt_p->head.start_id != SVETH_IPCC_START_ID)) {
		return 0;
	}
	*cmd_id_p = pkt_p->head.cmd_id;
	return (sveth_ipcc_cmd_t*)&pkt_p->cmd;
}



#endif	/* _SVETH_IPCC_LOCAL_H_ */


