/*
 *  sveth_mpool.h
 *
 *	The virtual ethernet driver for the SPiCa movie system.
 *
 *  Copyright (C) 2012 Panasonic Corporation. All Rights Reserved.
 *
 */

#ifndef	_SVETH_MPOOL_H_
#define	_SVETH_MPOOL_H_

#include <linux/init.h>
#include <linux/moduleparam.h>

#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/errno.h>
#include <linux/types.h>

#include <linux/list.h>

#include "sveth_util.h"

#define SVETH_MPOOL_ALIGN			(4)


/**
 *	data types
 */

struct sveth_mpool_qn_t;
struct sveth_mpool_qh_t;
struct sveth_mpool_que_t;


typedef void**					sveth_mpool_handle_t;
typedef void					(*pf_sveth_mpool_cl_handler)(void*);
typedef void					(*pf_sveth_mpool_tm_handler)(void*);


struct sveth_mpool_qn_t {
	struct list_head				this;
	
	size_t							size;
	void*							data_p;

	struct sveth_mpool_que_t*		que_p;
	
	struct timer_list				timer;
	pf_sveth_mpool_tm_handler		tm_handler;
};


struct sveth_mpool_qh_t {
	size_t						cnt;
	size_t						num;
	struct list_head			head;
};


struct sveth_mpool_que_t {
	rwlock_t					lock;
	
	struct sveth_mpool_qn_t*	node_p;
	void*						data_p;
	
	struct sveth_mpool_qh_t		free;
	struct sveth_mpool_qh_t		used;
};




/**
 *	inline
 */
/* handle to ptr */
static inline void* _sveth_mpool_handle_to_ptr(sveth_mpool_handle_t handle) {
	return *(void**)handle;
}



/**
 *	prototype
 */

/* private info (for debugging) */
void sveth_mpool_dump_priv_info(struct sveth_mpool_que_t* que_p);


/* init */
void sveth_mpool_init(struct sveth_mpool_que_t* que_p);

/* setup */
int sveth_mpool_setup(struct sveth_mpool_que_t* que_p, size_t num, size_t persize);

/* cleanup */
void sveth_mpool_cleanup(struct sveth_mpool_que_t* que_p, pf_sveth_mpool_cl_handler cl_handler);

/* allocate */
sveth_mpool_handle_t sveth_mpool_alloc(struct sveth_mpool_que_t* que_p);

/* free */
int sveth_mpool_free(struct sveth_mpool_que_t* que_p, sveth_mpool_handle_t handle);

/* set expiration */
int sveth_mpool_set_expiration(struct sveth_mpool_que_t* que_p, sveth_mpool_handle_t handle, pf_sveth_mpool_tm_handler tm_handler, unsigned long expires);

/* check valid or not */
int sveth_mpool_is_valid(struct sveth_mpool_que_t* que_p, sveth_mpool_handle_t handle);

/* check empty */
int sveth_mpool_is_empty(struct sveth_mpool_que_t* que_p);

#endif	/* _SVETH_MPOOL_H_ */


