/*
 *  sveth_que.h
 *
 *	The virtual ethernet driver for the SPiCa movie system.
 *
 *  Copyright (C) 2012 Panasonic Corporation. All Rights Reserved.
 *
 */

#ifndef	_SVETH_QUE_H_
#define	_SVETH_QUE_H_

#include <linux/init.h>
#include <linux/moduleparam.h>

#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/errno.h>
#include <linux/types.h>

#include <linux/list.h>

#include "sveth_util.h"

#define SVETH_PKTQUE_ALIGN			(4)


/**
 *	data types
 */

struct sveth_que_qn_t;
struct sveth_que_qh_t;
struct sveth_que_t;


typedef void					(*pf_sveth_que_cl_handler)(void*);
typedef void					(*pf_sveth_que_tm_handler)(void*);

struct sveth_que_qn_t {
	struct list_head			this;
	void*						data_p;

	struct sveth_que_t*			que_p;
	struct sveth_que_qh_t*		qh_p;

	struct timer_list			timer;
	pf_sveth_que_tm_handler		tm_handler;
};


struct sveth_que_qh_t {
	struct list_head			head;
	size_t						num;
};

struct sveth_que_t {
	rwlock_t					lock;

	struct sveth_que_qn_t*		node_p;

	struct sveth_que_qh_t		pool;
	struct sveth_que_qh_t		req;
};



/**
 *	prototype
 */

/* private info (for debugging) */
void sveth_que_dump_priv_info(struct sveth_que_t* que_p);

/* setup */
void sveth_que_init(struct sveth_que_t* que_p);

/* setup */
int sveth_que_setup(struct sveth_que_t* que_p, size_t num);

/* cleanup */
void sveth_que_cleanup(struct sveth_que_t* que_p, pf_sveth_que_cl_handler cl_handler);

/* check empty */
int sveth_que_is_empty(struct sveth_que_t* que_p);

/* check full */
int sveth_que_is_full(struct sveth_que_t* que_p);

/* get enqueue num */
size_t sveth_que_get_enqueue_num(struct sveth_que_t* que_p);

/* enqueue */
int sveth_que_enqueue(struct sveth_que_t* que_p, const void* data_p, pf_sveth_que_tm_handler tm_handler, unsigned long expires);

/* dequeue */
int sveth_que_dequeue(struct sveth_que_t* que_p, void** data_pp);


#endif	/* _SVETH_QUE_H_ */


