/*
 *  sveth_util.h
 *
 *	The virtual ethernet driver for the SPiCa movie system.
 *
 *  Copyright (C) 2012 Panasonic Corporation. All Rights Reserved.
 *
 */

#ifndef	_SVETH_UTIL_H_
#define	_SVETH_UTIL_H_

#include <linux/types.h>
#include <linux/bitrev.h>

#include "sveth_debug.h"
#include "net_event.h"
#include "sveth_debug_util.h"



#define _sveth_arrayof(array)			(sizeof(array) / sizeof((array)[0]))
#define _sveth_foreach(p, array)		for((p) = &(array)[0]; (p) < &(array)[_sveth_arrayof(array)]; (p)++)

#define _sveth_getval(_c)				(typeof(_c))(_c)
#define _sveth_clrval(_c)				((_c) = (typeof(_c))(0))
#define _sveth_setval(_c, _v)			((_c) = (typeof(_c))(_v))
#define _sveth_issetval(_c, _v)			((_c) == (typeof(_c))(_v))

#define _sveth_getflg(_c)				(typeof(_c))(_c)
#define _sveth_clrflg(_c, _f)			((_c) &= (typeof(_c))~(_f))
#define _sveth_setflg(_c, _f)			((_c) |= (typeof(_c))(_f))
#define _sveth_issetflg(_c, _f)			((_c) & (typeof(_c))(_f))

#define _sveth_index_nextof(_i, _n)		(((_i) < (typeof(_i))((_n) - 1)) ? (_i) + 1 : 0)
#define _sveth_index_prevof(_i, _n)		(((_i) > 0) ? (_i) - 1 : (typeof(_i))((_n) - 1))


/**
 *	for IEEE 802.3 CRC-32
 */
#define CRCPOLY2			(0xEDB88320UL)
static inline unsigned long _sveth_crc32(int n, unsigned char c[]) {
	int i;
	int bitcnt;
	unsigned long r;

	r = 0xFFFFFFFFUL;
	for(i = 0; i < n; i++) {
		r ^= c[i];
		for(bitcnt = 0; bitcnt < /* CHAR_BIT */ 8; bitcnt++) {
			if(r & 1) {
				r = (r >> 1) ^ CRCPOLY2;
			}
			else {
				r >>= 1;
			}
		}
	}
	return r ^ 0xFFFFFFFFUL;
}


#define sveth_calc_crc32_be(n, c)		bitrev32(_sveth_crc32((n), (c)))
#define sveth_calc_crc32_le(n, c)		_sveth_crc32((n), (c))



#endif	/* _SVETH_UTIL_H_ */


