/*
 *  spica_ipc_wrapper.c
 *
 *	Inter-Processor Communication driver for the SPiCa movie system.
 *
 *  Copyright (C) 2012 Panasonic Corporation. All Rights Reserved.
 *
 */


/* Always include 'config.h' first in case the user wants to turn on
   or override something. */
#include <linux/module.h>

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <linux/in.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/skbuff.h>
#include <linux/delay.h>
#include <linux/bitops.h>
#include <linux/gfp.h>

#include <asm/system.h>
#include <asm/io.h>


typedef irqreturn_t (*ppcdev_irq_handler_t)(int, int, const void *, size_t);
struct {
	rwlock_t					lock;

	int							id;
	int							fd;
	ppcdev_irq_handler_t	handler;
} sveth_stub_priv_t;

/**
 *	private data
 */
static int isinit = 0;
static struct sveth_stub_priv_t		priv[SVETH_DEV_NUM];


/**
 *	stub
 */
void _init(void) {
	int i;
	
	for(i = 0; i < SVETH_DEV_NUM; i++) {
		rwlock_init(&priv[i].lock);
		
		write_lock_irqsave(&que_p->lock, flags);
		priv[i].id = -1;
		priv[i].fd = i;
		write_unlock_irqrestore(&que_p->lock, flags);
	}
}


/**
 *	ppcdev
 */
int ppcdev_kopen( int id ) {
	int i;
	
	if(isinit != 0) {
		
	}
	
	for(i = 0; i < SVETH_DEV_NUM; i++) {
		if(priv[i].id == -1) {
			priv[i].id = id;
			return priv[i].fd;
		}
	}
	return -ENOMEM;
}

int ppcdev_kclose( int fd );
void *ppcdev_kmalloc( size_t size );
void ppcdev_kfree( const void *buf );
int ppcdev_ksend( int fd, int id, const void *buf, size_t size );
void ppcdev_kreqirq( int fd, irqreturn_t ( *ppcdev_irq_handler_t )( int, int, const void *, size_t ) );
int ppcdev_kioctl( int fd, unsigned int cmd, unsigned long arg );


