/*
 * Copyright (C) 2012, Panasonic Corporation.
 *                       All Rights Reserved.
 *
 */

#define INTERMEM_CONFIG_NUM	ARRAY_SIZE( intermem_configuration )

typedef struct _INTERMEM_GROUP{
	const INTERMEM_CONFIG* nodeConfig;	/*_ 롼פ _*/
	u8*		topAddress;					/*_ 롼פǰƬɥ쥹(CPU) _*/
	u8*		topAddress_dma;				/*_ 롼פǰƬɥ쥹(DMA) _*/
	u8*		bottomAddress;				/*_ 롼פǰκǸɥ쥹(CPU) _*/
	
	int		waitingNum;					/*_ ŻԤΡɤο _*/
	int*	waitingStack;				/*_ ŻԤå _*/
	
	u8*		workingCheckTable;			/*_ Żơ֥ _*/
} INTERMEM_GROUP;

typedef struct _INTERMEM_TOP{
	INTERMEM_GROUP	groups[INTERMEM_CONFIG_NUM];	/*_ 롼פμ _*/
	u8*				topAddress;						/*_ INTERMEMƬɥ쥹(CPU) _*/
	spinlock_t		lock;
} INTERMEM_TOP;

#define INTERMEM_LOCK( EHCI )										\
	{	unsigned long lockflags;									\
		spin_lock_irqsave( &(EHCI)->intermem->lock, lockflags );

#define INTERMEM_UNLOCK( EHCI )											\
		spin_unlock_irqrestore( &(EHCI)->intermem->lock, lockflags );	\
	}


/*_ 롼פ _*/
static inline void freeGroups( INTERMEM_GROUP* gr )
{
	int i;
	
	if( !gr ) return;
	
	for( i=0 ; i < INTERMEM_CONFIG_NUM ; i++ ){
		if( gr[i].waitingStack )		kfree( gr[i].waitingStack );
		if( gr[i].workingCheckTable )	kfree( gr[i].workingCheckTable );
	}
}

/*_ ס _*/
static void usbh_sharemem_finish( struct ehci_hcd *ehci )
{
	INTERMEM_TOP* top;
	
	top = ehci->intermem;
	if( !top ) return;
	ehci->intermem = NULL;
	
/*_ @LINUXHOST:V0302nx[11/11/11] IOREMAP򥳥եǤ褦ѹ _*/
#ifdef CONFIG_USB_PANASONIC_EHCI_IOREMAP
	iounmap( top->topAddress );
#endif
	freeGroups( top->groups );
	kfree( top );
}


/*_ 롼פݤ _*/
static inline int allocateGroups( INTERMEM_GROUP* gr, u8* address, u8* dma )
{
	size_t usedBufferSize;
	int i, j, num;
	
	usedBufferSize = 0;
	
	for( i=0 ; i < INTERMEM_CONFIG_NUM ; i++, gr++ ){
		gr->nodeConfig = &intermem_configuration[i];
		num = gr->nodeConfig->num;
		gr->topAddress =		address	+ usedBufferSize;
		gr->topAddress_dma =	dma		+ usedBufferSize;
		usedBufferSize +=		num * gr->nodeConfig->size;
		gr->bottomAddress =		address	+ usedBufferSize -1;
		if( INTERMEM_SIZE < usedBufferSize )
			return -1;
		
		gr->waitingStack = kzalloc( sizeof(int) * num, GFP_ATOMIC );
		if( !gr->waitingStack ) return -1;
		
		gr->workingCheckTable = kzalloc( sizeof(u8) * num, GFP_ATOMIC );
		if( !gr->workingCheckTable ) return -1;
		
		/*_ Que _*/
		gr->waitingNum = num;
		for( j=0 ; j < num ; j++ ) gr->waitingStack[j] = num -j -1;
	}
	return 0;
}

/*_ סݤԤ _*/
static int usbh_sharemem_init( struct ehci_hcd *ehci )
{
	u8* address;
	u8* dma;
	
	assert( ehci->intermem == NULL );
	ehci->intermem = kzalloc( sizeof(INTERMEM_TOP), GFP_ATOMIC );
	if( !ehci->intermem ) return -1;
	spin_lock_init( &ehci->intermem->lock );
	dma = address = (u8*)( (u32)ehci_to_hcd(ehci)->rsrc_start ) + INTERMEM_OFFSET;	/*_ 64ӥåȤľܥ㥹Ȥȥ˥ _*/

/*_ @LINUXHOST:V0302nx[11/11/11] IOREMAP򥳥եǤ褦ѹ _*/
#ifdef CONFIG_USB_PANASONIC_EHCI_IOREMAP
	address = ioremap_nocache( (resource_size_t)address, INTERMEM_SIZE );
	assert( address != NULL );
#endif
	memset( address, 0x00, INTERMEM_SIZE );
	ehci->intermem->topAddress = address;
	
	if( allocateGroups( ehci->intermem->groups, address, dma ) ){
		printk( "%s: allocateGroups fail.\n", __FUNCTION__ );
		usbh_sharemem_finish( ehci );
		return -1;
	}
	
	return 0;
}


/*_  ꤷơŬ롼פ롣 _*/
static inline INTERMEM_GROUP* getSuitableGroupFromSize( INTERMEM_TOP* top, size_t size )
{
	int i;
	INTERMEM_GROUP* gr;
	
	if( top == NULL ) return NULL;
	
	for( i = INTERMEM_CONFIG_NUM-1 ; 0 <= i ; i-- ){	/*_ 1024ΥХåե˻äƤˡդõ _*/
		gr = &top->groups[i];
		if( gr->waitingNum != 0 && size <= gr->nodeConfig->size ) return gr;
	}
	return NULL;
}

/*_  ס뤫ݤ _*/
static void* usbh_malloc_share( struct ehci_hcd *ehci, size_t size, dma_addr_t* dma )
{
	int index, offset;
	INTERMEM_GROUP* gr;
	void* ret;
	
	ret = NULL;
	
	INTERMEM_LOCK( ehci );
	gr = getSuitableGroupFromSize( ehci->intermem, size );
	if( gr ){
		index = gr->waitingStack[ gr->waitingNum -1 ];	/*_ ŻԤåΡɤ _*/
		gr->waitingNum--;								/*_ ŻԤͿ _*/
		assert( gr->workingCheckTable[ index ] == 0 );	/*_ ˻Ż򤷤ƤȤ _*/
		gr->workingCheckTable[index] = 1;				/*_ Ż򥻥å _*/
		
		offset = gr->nodeConfig->size * index;			/*_ ΡɤֹȥΡɤΥ顢롼ΥեåȤ׻ _*/
		ret = (void*) ( gr->topAddress + offset );		/*_ 롼פƬɥ쥹ȥ롼ΥեåȤ­Ρɤб륢ɥ쥹׻ _*/
		if( dma ) *dma = (dma_addr_t) ( gr->topAddress_dma + offset );	/*_ DMAɥ쥹Ʊͤη׻ _*/
	}
	INTERMEM_UNLOCK( ehci );
	
/*_ printk("%s:size=%u, ret=%p, dma=%08x\n", __FUNCTION__, size, ret, (dma==NULL)?NULL:*dma ); _*/
	return ret;
}


/*_  ꥢɥ쥹ޤǤ륰롼פ֤ _*/
static inline INTERMEM_GROUP* getGroupFromAddress( INTERMEM_TOP* top, void* ptr )
{
	int i;
	INTERMEM_GROUP* gr;
	
	if( top == NULL ) return NULL;
	
	for( i = INTERMEM_CONFIG_NUM-1 ; 0 <= i ; i-- ){	/*_ 1024ΥХåե˻äƤˡդõ _*/
		gr = &top->groups[i];
		if( gr->topAddress <= (u8*)ptr && (u8*)ptr <= gr->bottomAddress ) return gr;
	}
	return NULL;
}

/*_  usbh_malloc_shareǳݤ _*/
static void usbh_free_share( struct ehci_hcd *ehci, void* ptr )
{
	int index, offset;
	INTERMEM_GROUP* gr;
	
	INTERMEM_LOCK( ehci );
	gr = getGroupFromAddress( ehci->intermem, ptr );
	if( gr ){
		offset = ( (u8*)ptr - gr->topAddress );		/*_ 롼⥪եå _*/
		index  = offset / gr->nodeConfig->size;		/*_ 롼⥪եåȤȥΡɤΥ顢ǥå׻ _*/
		assert( gr->workingCheckTable[index] );		/*_ Ż򤷤Ƥʤ֤ǤΤϤ _*/
		
		gr->workingCheckTable[ index ] = 0;			/*_ ŻλϿ _*/
		gr->waitingStack[ gr->waitingNum ] = index;	/*_ ΡɤŻԤå᤹ _*/
		gr->waitingNum++;							/*_ ŻԤͿ _*/
	}
	INTERMEM_UNLOCK( ehci );
}

