/*
 * Copyright (C) 2012, Panasonic Corporation 
 */

#if defined(CONFIG_USB_PANASONIC_PRO4R)

#ifdef CONFIG_USB_IGNORE_CERR
static u8	usb_xhci_target = 1;
#endif

/*_------------------------------------------------------------------------
	USB3(XHCI) SoCレジスタ定義
------------------------------------------------------------------------_*/
#define XHCI_SOC_RSTCTRL_REG      (0xE1842000)    /*_  SoC Reset Control Register (USB0) _*/
#define XHCI_SOC_RSTCTRL2_REG     (0xE1842004)    /*_  SoC Reset Control Register (USB1) _*/

#define XHCI_RESET_CTL_REG_0      (0xE5B00040)    /*_  Reset Control Register (USB0) _*/
#define XHCI_RESET_CTL_REG_1      (0xE5D00040)    /*_  Reset Control Register (USB1) _*/

#define XHCI_USBPHY3_CTRL_REG     (0xDF800510)    /*_  PHY3 set register             _*/
#define XHCI_USBPHY3_CTRL2_REG    (0xDF800514)    /*_  PHY3 set register             _*/
#define XHCI_USBPHY4_CTRL_REG     (0xDF800518)    /*_  PHY4 set register             _*/
#define XHCI_USBPHY4_CTRL2_REG    (0xDF80051C)    /*_  PHY4 set register             _*/

#define XHCI_USB3_VBUS_CONTROL_OFFSET   (0x00100000)	/*_ USB3 VBUS OFFSET FROM cap_reg _*/
#define XHCI_USB3_VBUS_CONTROL_REG_0	(0xE5B00000)	/*_ USB3 VBUS Control _*/
#define XHCI_USB3_VBUS_CONTROL_REG_1	(0xE5D00000)	/*_ USB3 VBUS Control _*/

#define XHCI_USB3_VBUS_DRVVBUS_REG_EN   (1<<3)		/*_ VBUS ON Controled by EN _*/
#define XHCI_USB3_VBUS_DRVVBUS_REG      (1<<4)		/*_ VBUS ON _*/

#define XHCI_USB3_TESTI_REG_0     (0xE5B00010)    /*_ SS-PHY set register            _*/
#define XHCI_USB3_TESTO_REG_0     (0xE5B00014)    /*_ SS-PHY dummy register            _*/
#define XHCI_USB3_TESTI_REG_1     (0xE5D00010)    /*_ SS-PHY set register            _*/
#define XHCI_USB3_TESTO_REG_1     (0xE5D00014)    /*_ SS-PHY dummy register            _*/


/*_ @LINUXHOST:V0303nd[11/12/12] xHCI 割り込み番号変更 _*/
#define XHCI_BASE_ADR_0           (0xE5A00000)    /*_ host #0 XHCI Base Address	_*/
//#define XHCI_IRQ_NUM_0            (119)           /*_ host #0 XHCI IRQ NUM(IPP)	_*/
#define XHCI_IRQ_NUM_0            (166)           /*_ host #0 XHCI IRQ NUM(ARM)	_*/

#define XHCI_BASE_ADR_1           (0xE5C00000)    /*_ host #1 XHCI Base Address	_*/
//#define XHCI_IRQ_NUM_1            (202)           /*_ host #1 XHCI IRQ NUM(IPP)	_*/
#define XHCI_IRQ_NUM_1            (169)           /*_ host #1 XHCI IRQ NUM(ARM)	_*/

#define XHCI_MEM_LEN              (0x7FFF)

static const struct resource host_res[][2] = {
	{/*_  host #0 _*/
		{
			.start	= XHCI_BASE_ADR_0,
			.end	= XHCI_BASE_ADR_0 + XHCI_MEM_LEN - 1,
			.flags	= IORESOURCE_MEM,
		}, {
			.start	= XHCI_IRQ_NUM_0,
			.flags	= IORESOURCE_IRQ,
		},
	},
	{/*_  host #1 _*/
		{
			.start	= XHCI_BASE_ADR_1,
			.end	= XHCI_BASE_ADR_1 + XHCI_MEM_LEN - 1,
			.flags	= IORESOURCE_MEM,
		}, {
			.start	= XHCI_IRQ_NUM_1,
			.flags	= IORESOURCE_IRQ,
		},
	},
};



/*_---------------------------------------------------------------------------
     ハードウェア開始(初期化)処理
---------------------------------------------------------------------------_*/
static inline void xhci_panasonic_peripheryhw_init( void )
{


	u32 tmp;
	u32 caplen;
	int i;
	
	/*_ USBブロック リセット解除 _*/

	/*_ USB0 (RSTCTRL) _*/
	tmp = readl((void*) XHCI_SOC_RSTCTRL_REG);
	tmp |= 0x00020000;
	writel( tmp, (void*)XHCI_SOC_RSTCTRL_REG );

	/*_ USB1 (RSTCTRL2) _*/
	tmp = readl((void*) XHCI_SOC_RSTCTRL2_REG);
	tmp |= 0x00020000;
	writel( tmp, (void*)XHCI_SOC_RSTCTRL2_REG );


	/*_ vbusvalid、drvvbus端子設定 _*/
#if 0
	/*_ 他ブロックで実施されていない場合、有効に _*/

	/*_ PINCTRL67 _*/
	tmp = readl((void*) 0x5F80110c);
	tmp &= 0x0000ffff;
	tmp += 0x09090000;
	writel( tmp, (void*)0x5F80110c );

	/*_ PINCTRL68 _*/
	tmp = readl((void*) 0x5F801110);
	tmp &= 0x00000000;
	tmp += 0x09090909;
	writel( tmp, (void*)0x5F801110 );

	/*_ PINCTRL69 _*/
	tmp = readl((void*) 0x5F80111c);
	tmp &= 0x0000ffff;
	tmp += 0x09090000;
	writel( tmp, (void*)0x5F80111c );
#endif


	/*_ PHYクロック供給設定 _*/
#if 0
	/*_ 他ブロックで実施されていない場合、有効に _*/

	/*_ internal GPLL mode 24.576MHz _*/

	/*_ GPLL stop _*/
	writel( 0x00000000, (void*)0x61841194 );
	while(0x0 != readl((void*) 0x61841194)){ msleep(1) };

	/*_ GPLLSRCSELEN=1'b1,GPLLSRCSEL=2'b10 _*/
	writel( 0x00000012, (void*)0x61841194 );

	/*_ CK25OSEL=1'b1 _*/
	writel( 0x00000001, (void*)0x61842260 );

	/*_ GPLL run _*/
	writel( 0x00000001, (void*)0x61841194 );
	while(0x3 != readl((void*) 0x61841194)){ msleep(1) };
#endif


	/*_ HS PHY設定 _*/
        writel( 0x05142400, (void*)XHCI_USBPHY3_CTRL_REG );     /*_PHY3 _*/
        writel( 0x00000007, (void*)XHCI_USBPHY3_CTRL2_REG );    /*_PHY3 _*/
        writel( 0x05142400, (void*)XHCI_USBPHY4_CTRL_REG );     /*_PHY4 _*/
        writel( 0x00010010, (void*)XHCI_USBPHY4_CTRL2_REG );    /*_PHY4 _*/

	/*_ Control the VBUS by xHCI driver _*/
	tmp = readl((void*) XHCI_USB3_VBUS_CONTROL_REG_0);
	tmp |= 0x00000018;	/*_ 連動解除およびVBUSのON  _*/
	writel( tmp, (void*)XHCI_USB3_VBUS_CONTROL_REG_0);

	tmp = readl((void*) XHCI_USB3_VBUS_CONTROL_REG_1);
	tmp |= 0x00000018;
	writel( tmp, (void*)XHCI_USB3_VBUS_CONTROL_REG_1);

	/*_ Wait for the VBUS is stable _*/
	msleep(30);

	/*_ IOMMU &  LINK リセット解除 _*/

	/*_ USB0 _*/
	tmp = readl((void*) XHCI_RESET_CTL_REG_0);
	tmp |= 0x00000030;
	writel( tmp, (void*)XHCI_RESET_CTL_REG_0 );

	/*_ USB1 _*/
	tmp = readl((void*) XHCI_RESET_CTL_REG_1);
	tmp |= 0x00000030;
	writel( tmp, (void*)XHCI_RESET_CTL_REG_1 );


	/*_ PLL安定待ち(必要なし?) _*/
	//msleep(1);

//	PHYの書き込み確認用
//	writel( 0x0000031e, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	for(i = 0;i < 10;i++){
//		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	}
//	writel( 0x0000031f, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	for(i = 0;i < 10;i++){
//		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	}
//	writel( 0x0000031e, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	for(i = 0;i < 10;i++){
//		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	}
//	writel( 0x00000018, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	for(i = 0;i < 10;i++){
//		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	}
//	tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	printk(KERN_ERR "PHY Resister : Before : init = 0x%x\n",tmp);
//
//	writel( 0x0000021e, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	for(i = 0;i < 10;i++){
//		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	}
//	writel( 0x0000021f, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	for(i = 0;i < 10;i++){
//		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	}
//	writel( 0x0000021e, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	for(i = 0;i < 10;i++){
//		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	}

	/*_ SS PHY設定 _*/
	/*_ SS PHY設定はReset時にクリアされるため、xhci.cのxhci_resetでも再設定している _*/
	/*_ 設定値はハード部門から連絡 _*/
	writel( 0x00000206, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	/*_ 各書き込みの間には80ns以上の間隔が必要 _*/
	/*_ ダミーリードして時間調整を行う場合は読み出しx10で調整する _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x00000207, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x00000206, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x0000008e, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x0000008f, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x0000008e, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x0000027c, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x0000027d, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x0000027c, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x000002b8, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x000002b9, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x000002b8, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x00000102, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x00000103, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x00000102, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x0000022a, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x0000022b, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x0000022a, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x000000ac, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x000000ad, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x000000ac, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x00000100, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x00000101, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x00000100, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x0000009a, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x0000009b, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
	for(i = 0;i < 10;i++){
		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
	}
	writel( 0x0000009a, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	writel( 0xXXXXXXXX, (void*)XHCI_USB3_TESTI_REG_1 );	    /*_ SS-PHY(USB1) _*/

//	PHYの書き込み確認用
//	writel( 0x0000031e, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	for(i = 0;i < 10;i++){
//		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	}
//	writel( 0x0000031f, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	for(i = 0;i < 10;i++){
//		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	}
//	writel( 0x0000031e, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	for(i = 0;i < 10;i++){
//		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	}
//	writel( 0x00000018, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	for(i = 0;i < 10;i++){
//		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	}
//	tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	printk(KERN_ERR "PHY Resister : After : init = 0x%x\n",tmp);
//
//	writel( 0x0000021e, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	for(i = 0;i < 10;i++){
//		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	}
//	writel( 0x0000021f, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	for(i = 0;i < 10;i++){
//		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	}
//	writel( 0x0000021e, (void*)XHCI_USB3_TESTI_REG_0 );		/*_ SS-PHY(USB0) _*/
//	for(i = 0;i < 10;i++){
//		tmp = readl((void*)XHCI_USB3_TESTO_REG_0);
//	}

	/*_ check HCRST bit (USBCMD Reg) _*/
	/*_ USB0 _*/
	caplen = 0xFF & readl((void*) XHCI_BASE_ADR_0);
	tmp = readl((void*) (XHCI_BASE_ADR_0 + caplen) );
	if(tmp & 0x2) 	printk(KERN_ERR "Usb3-0 init error \n");

	/*_ USB1 _*/
	caplen = 0xFF & readl((void*) XHCI_BASE_ADR_1);
	tmp = readl((void*) (XHCI_BASE_ADR_1 + caplen) );
	if(tmp & 0x2) 	printk(KERN_ERR "Usb3-1 init error \n");

}

/*_---------------------------------------------------------------------------
     ハードウェア終了処理
---------------------------------------------------------------------------_*/
static inline void xhci_panasonic_peripheryhw_end( void )
{
	u32 tmp;

	/*_ IOMMU &  LINK リセット _*/

	/*_ USB0 _*/
	tmp = readl((void*) XHCI_RESET_CTL_REG_0);
	tmp &= ~0x00000030;
	writel( tmp, (void*)XHCI_RESET_CTL_REG_0 );

	/*_ USB1 _*/
	tmp = readl((void*) XHCI_RESET_CTL_REG_1);
	tmp &= ~0x00000030;
	writel( tmp, (void*)XHCI_RESET_CTL_REG_1 );

	/*_ USBブロック リセット _*/

	/*_ USB0 (RSTCTRL) _*/
	tmp = readl((void*) XHCI_SOC_RSTCTRL_REG);
	tmp &= ~0x00020000;
	writel( tmp, (void*)XHCI_SOC_RSTCTRL_REG );

	/*_ USB1 (RSTCTRL2) _*/
	tmp = readl((void*) XHCI_SOC_RSTCTRL2_REG);
	tmp &= ~0x00020000;
	writel( tmp, (void*)XHCI_SOC_RSTCTRL2_REG );
}

#endif /* CONFIG_USB_PANASONIC_PRO4R */
