#ifndef _ICCDEF_H_
#define _ICCDEF_H_

#include "icc_cfg.h"
#include "icc_if.h"

#define ICC_VER_MAJOR		1
#define ICC_VER_MINOR		2
#define ICC_VER_RELEASE		3

#define ICC_VERSION_DEFINE(a, b, c)	\
	(((a) << 12) | ((b) << 6) | ((c) << 0))

#define ICC_VERSION	\
	ICC_VERSION_DEFINE(ICC_VER_MAJOR, ICC_VER_MINOR, ICC_VER_RELEASE)

/* The variable types definitions */
#if defined(ICC_LINUX)
#ifdef __KERNEL__
#include <linux/types.h>
#else
#include <stdint.h>
#endif
#else /* !ICC_LINUX */
#ifndef _icc_icsm_typedef_
#define _icc_icsm_typedef_
typedef unsigned char	uint8_t;
typedef unsigned short	uint16_t;
typedef unsigned int	uint32_t;
#endif
#ifndef NULL
#define NULL	0
#endif
#endif /* !ICC_LINUX */

#if defined(__CC__)
#ifndef inline
#define inline
#endif
#elif defined(__CC_ARM)
#ifndef inline
#define inline __inline
#endif
#endif

/* The CPU number definitions */
#define ICC_CPU0	0
#define ICC_CPU1	1
#define ICC_CPU2	2
#define ICC_CPU3	3

/* CPU numbers in the system  */
#define ICC_CPU_NUM	4

/* icc_basecb_t->state[] definitions */
#define ICC_STATE_INIT	0x00
#define ICC_STATE_START	0x01
#define ICC_STATE_RUN	0x02
#define ICC_STATE_STOP	0x04
#define ICC_STATE_ABORT	0xe0

/* FIFO structure definitions */
typedef struct icc_basecb {
	uint16_t version;
	uint8_t lock;
	uint8_t reserved;
	uint8_t state[ICC_CPU_NUM];
	uint32_t idtable[ICC_CPU_NUM][ICC_CPU_ID_MAX / 32 + 1];
} icc_basecb_t;

typedef struct icc_fifocb {
	uint32_t length;
	uint16_t roffset;
	uint16_t woffset;
} icc_fifocb_t;

typedef struct icc_packetcb {
	uint16_t dstid;
	uint16_t length;
} icc_packetcb_t;

#define ICC_PACKET_MESSAGE(packetcb)	\
	((void*)((unsigned long)(packetcb) + sizeof(icc_packetcb_t)))

#define ICC_PACKET_MAX_LEN	\
	(ICC_MESSAGE_MAX_LEN + sizeof(icc_packetcb_t))

#define ALIGNUP(a, b)	(((a) + (b) - 1) & ~((b) - 1))

/*
  CPU Communication Destination ID
    bit 15-14: CPU
               - 00: CPU0
               - 01: CPU1
               - 10: CPU2
               - 11: CPU3
    bit 13-0 : Destination
 */
#define ICC_IDBASE_SHIFT	14
#define ICC_IDBASE_CPU0		(0x0 << ICC_IDBASE_SHIFT)
#define ICC_IDBASE_CPU1		(0x1 << ICC_IDBASE_SHIFT)
#define ICC_IDBASE_CPU2		(0x2 << ICC_IDBASE_SHIFT)
#define ICC_IDBASE_CPU3		(0x3 << ICC_IDBASE_SHIFT)
#define ICC_IDBASE_MASK		(0x3 << ICC_IDBASE_SHIFT)
#define ICC_ID(cpu, dst)	(((cpu) << ICC_IDBASE_SHIFT) | (dst))

static inline int icc_id_owner(unsigned short id)
{
	switch (id & ICC_IDBASE_MASK) {
	case ICC_IDBASE_CPU0:
		return ICC_CPU0;
	case ICC_IDBASE_CPU1:
		return ICC_CPU1;
	case ICC_IDBASE_CPU2:
		return ICC_CPU2;
	case ICC_IDBASE_CPU3:
		return ICC_CPU3;
	default:
		break;
	}
	return ICC_E_PAR;
}

static inline int icc_check_id_range(unsigned short id)
{
	switch (icc_id_owner(id)) {
	case ICC_CPU0:
		if (ICC_IDBASE_CPU0 < id && \
		    id <= (ICC_IDBASE_CPU0 + ICC_CPU_ID_MAX)) {
			return 0;
		}
		break;
	case ICC_CPU1:
		if (ICC_IDBASE_CPU1 < id && \
		    id <= (ICC_IDBASE_CPU1 + ICC_CPU_ID_MAX)) {
			return 0;
		}
		break;
	case ICC_CPU2:
		if (ICC_IDBASE_CPU2 < id && \
		    id <= (ICC_IDBASE_CPU2 + ICC_CPU_ID_MAX)) {
			return 0;
		}
		break;
	case ICC_CPU3:
		if (ICC_IDBASE_CPU3 < id && \
		    id <= (ICC_IDBASE_CPU3 + ICC_CPU_ID_MAX)) {
			return 0;
		}
		break;
	default:
		break;
	}
	return ICC_E_PAR;

}

#endif /* _ICCDEF_H_ */
