/*
 * Copyright (c) 2013-     Panasonic Corporation
 * $Id$
 */
/**
   @file    Usbhd_IccEvent.h
   @author  
   @brief   USB Host Controller Driver / Icc Event
*/

#ifndef _USBHD_ICC_EVENT_H_
#define	_USBHD_ICC_EVENT_H_


/*---------------------------------------------------------------------------*/
#include <icc_cfg.h>
#define USBHD_ICC_PAYLOAD_SIZE			ICC_MESSAGE_MAX_LEN

/*---------------------------------------------------------------------------*/
/**
 * command
 */
#define USBHD_ICC_CMD_INVALID			0x00
#define USBHD_ICC_CMD_START				0x01	/* Request, Response */
#define USBHD_ICC_CMD_STOP				0x02	/* Request, Response */
#define USBHD_ICC_CMD_CTRL				0x03	/* Request, Response */
#define USBHD_ICC_CMD_BULK				0x04	/* Request, Response */
#define USBHD_ICC_CMD_CLEARHALT			0x08	/* Request, Response */
#define USBHD_ICC_CMD_TEST				0x0F	/* Request, Response */
#define USBHD_ICC_CMD_DETACH			0x80	/* Notify */
#define USBHD_ICC_CMD_ATTACH			0x81	/* Notify */
#define USBHD_ICC_CMD_EMERGENCY			0x8F	/* Notify */

/**
 * subcommand
 */
/** test mode */
#define USBHD_ICC_TEST_NAK				1		/* @see: USB_TEST_NAK */
#define USBHD_ICC_TEST_J				2		/* @see: USB_TEST_J */
#define USBHD_ICC_TEST_K				3		/* @see: USB_TEST_K */
#define USBHD_ICC_TEST_PACKET			4		/* @see: USB_TEST_PACKET */
#define USBHD_ICC_TEST_RESUME			5		/* @see: USB_TEST_RESUME */
#define USBHD_ICC_TEST_DESC				6		/* @see: USB_TEST_DESC */
#define USBHD_ICC_TEST_FEATURE			7		/* @see: USB_TEST_FEATURE */
#define USBHD_ICC_TEST_DESC_RETRY		8		/* @see: USB_TEST_DESC_RETRY */
#define USBHD_ICC_TEST_FEATURE_RETRY	9		/* @see: USB_TEST_FEATURE_RETRY */

/** transfer direction */
#define USBHD_ICC_DIR_HOST2DEV			0		/* @see: USB_HOST2DEV */
#define USBHD_ICC_DIR_DEV2HOST			1		/* @see: USB_DEV2HOST */

/**
 * status
 */
#define USBHD_ICC_STS_COMPLETION		0		/* @see: USBHD_ERR_COMPLETION */
#define USBHD_ICC_STS_STALLED			1		/* @see: USBHD_ERR_STALLED */
#define USBHD_ICC_STS_CANCELLED			2		/* @see: USBHD_ERR_CANCELLED */
#define USBHD_ICC_STS_TIMEOUT			3		/* @see: USBHD_ERR_TIMEOUT */
#define USBHD_ICC_STS_INVAL				4		/* @see: USBHD_ERR_INVAL */
#define USBHD_ICC_STS_OTHER				98		/* @see: USBHD_ERR_OTHER */
#define USBHD_ICC_STS_NOOPERATION		99		/* @see: USBHD_ERR_NOOPERATION */

/**
 * speed
 */
#define USBHD_ICC_SPEED_UNKNOWN			0		/* @see: USB_SPEED_UNKNOWN */
#define USBHD_ICC_SPEED_LOW				1		/* @see: USB_SPEED_LOW */
#define USBHD_ICC_SPEED_FULL			2		/* @see: USB_SPEED_FULL */
#define USBHD_ICC_SPEED_HIGH			3		/* @see: USB_SPEED_HIGH */
#define USBHD_ICC_SPEED_VARIABLE		4		/* @see: USB_SPEED_VARIABLE */
#define USBHD_ICC_SPEED_SUPER			5		/* @see: USB_SPEED_SUPER */


/*---------------------------------------------------------------------------*/
#pragma pack(1) /* Alignment: 1byte */

/**
 * common
 */
typedef struct {
	unsigned char	length;
	unsigned char	command;
	unsigned char	status;						/* Response, Notify */
} USBHD_ICC_COMMON_HEADER;

/**
 * Request
 */
/** Header  */
typedef struct {
	unsigned char	length;
	unsigned char	command;
	unsigned char	subcommand;
	unsigned char	speed;
	unsigned char	address;
	unsigned char	endpoint_number;
	unsigned short	submit_timeout;				/* little endian */
	unsigned long	buff_addr;					/* little endian */
	unsigned long	buff_size;					/* little endian */
} USBHD_ICC_REQUEST_HEADER;

/** Start */
typedef struct {
	USBHD_ICC_REQUEST_HEADER	header;
} USBHD_ICC_REQUEST_START;

/** Stop */
typedef struct {
	USBHD_ICC_REQUEST_HEADER	header;
} USBHD_ICC_REQUEST_STOP;

/** Test */
typedef struct {
	USBHD_ICC_REQUEST_HEADER	header;
} USBHD_ICC_REQUEST_TEST;

/** Clear Halt */
typedef struct {
	USBHD_ICC_REQUEST_HEADER	header;
} USBHD_ICC_REQUEST_CLEARHALT;

/** Ctrl */
typedef struct {
	USBHD_ICC_REQUEST_HEADER	header;
	unsigned char payload[USBHD_ICC_PAYLOAD_SIZE - sizeof(USBHD_ICC_REQUEST_HEADER)];
} USBHD_ICC_REQUEST_CTRL;

/** Bulk */
typedef struct {
	USBHD_ICC_REQUEST_HEADER	header;
	unsigned char payload[USBHD_ICC_PAYLOAD_SIZE - sizeof(USBHD_ICC_REQUEST_HEADER)];
} USBHD_ICC_REQUEST_BULK;



/**
 * Response
 */
typedef struct {
	unsigned char	length;
	unsigned char	command;
	unsigned char	status;
	unsigned char	speed;
	unsigned char	address;
	unsigned char	endpoint_number;
	unsigned short	submit_timeout;				/* little endian */
	unsigned long	buff_addr;					/* little endian */
	unsigned long	buff_size;					/* little endian */
} USBHD_ICC_RESPONSE_HEADER;

/** Start */
typedef struct {
	USBHD_ICC_RESPONSE_HEADER	header;
} USBHD_ICC_RESPONSE_START;

/** Stop */
typedef struct {
	USBHD_ICC_RESPONSE_HEADER	header;
} USBHD_ICC_RESPONSE_STOP;

/** Test */
typedef struct {
	USBHD_ICC_RESPONSE_HEADER	header;
} USBHD_ICC_RESPONSE_TEST;

/** Clear Halt */
typedef struct {
	USBHD_ICC_RESPONSE_HEADER	header;
} USBHD_ICC_RESPONSE_CLEARHALT;

/** Ctrl */
typedef struct {
	USBHD_ICC_RESPONSE_HEADER	header;
	unsigned char payload[USBHD_ICC_PAYLOAD_SIZE - sizeof(USBHD_ICC_RESPONSE_HEADER)];
} USBHD_ICC_RESPONSE_CTRL;

/** Bulk */
typedef struct {
	USBHD_ICC_RESPONSE_HEADER	header;
	unsigned char payload[USBHD_ICC_PAYLOAD_SIZE - sizeof(USBHD_ICC_RESPONSE_HEADER)];
} USBHD_ICC_RESPONSE_BULK;


/**
 * Notify
 */
typedef struct {
	unsigned char	length;
	unsigned char	command;
	unsigned char	status;
	unsigned char	speed;
	unsigned char	address;
	unsigned char	endpoint_number;
	unsigned char	reserved1;
	unsigned char	reserved2;
	unsigned long	buff_addr;					/* little endian */
	unsigned long	buff_size;					/* little endian */
} USBHD_ICC_NOTIFY_HEADER;

/** Attach */
typedef struct {
	USBHD_ICC_NOTIFY_HEADER	header;
	unsigned char payload[USBHD_ICC_PAYLOAD_SIZE - sizeof(USBHD_ICC_NOTIFY_HEADER)];
} USBHD_ICC_NOTIFY_ATTACH;

/** Detach */
typedef struct {
	USBHD_ICC_NOTIFY_HEADER	header;
} USBHD_ICC_NOTIFY_DETACH;

/** Emergency */
typedef struct {
	USBHD_ICC_NOTIFY_HEADER	header;
} USBHD_ICC_NOTIFY_EMERGENCY;


/**
 * Event
 */
typedef union {
	unsigned char	buffer[USBHD_ICC_PAYLOAD_SIZE];
	
	union {
		USBHD_ICC_COMMON_HEADER			header;
	} common;
	
	union {
		USBHD_ICC_REQUEST_START			start;
		USBHD_ICC_REQUEST_STOP			stop;
		USBHD_ICC_REQUEST_TEST			test;
		USBHD_ICC_REQUEST_CLEARHALT		clearhalt;
		USBHD_ICC_REQUEST_CTRL			ctrl;
		USBHD_ICC_REQUEST_BULK			bulk;
	} request;
	
	union {
		USBHD_ICC_RESPONSE_START		start;
		USBHD_ICC_RESPONSE_STOP			stop;
		USBHD_ICC_RESPONSE_TEST			test;
		USBHD_ICC_RESPONSE_CLEARHALT	clearhalt;
		USBHD_ICC_RESPONSE_CTRL			ctrl;
		USBHD_ICC_RESPONSE_BULK			bulk;
	} response;
	
	union {
		USBHD_ICC_NOTIFY_ATTACH			attach;
		USBHD_ICC_NOTIFY_DETACH			detach;
		USBHD_ICC_NOTIFY_EMERGENCY		emergency;
	} notify;
	
} USBHD_ICC_EVENT;

#pragma pack() /* Alignment: revert */

#endif	/* _USBHD_ICC_EVENT_H_ */
